/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Extended description about the product required for installing it into Azure
 * Stack.
 *
 */
class ExtendedProduct {
  /**
   * Create a ExtendedProduct.
   * @member {string} [galleryPackageBlobSasUri] The URI to the .azpkg file
   * that provides information required for showing product in the gallery.
   * @member {string} [productKind] Specifies the kind of the product
   * (virtualMachine or virtualMachineExtension).
   * @member {string} [computeRole] Specifies kind of compute role inclided in
   * the package. Possible values include: 'None', 'IaaS', 'PaaS'
   * @member {boolean} [isSystemExtension] Specifies if product is a Virtual
   * Machine Extension.
   * @member {string} [uri] The URI.
   * @member {boolean} [supportMultipleExtensions] Indicates if specified
   * product supports multiple extensions.
   * @member {string} [version] Specifies product version.
   * @member {string} [vmOsType] Specifies operating system used by the
   * product. Possible values include: 'None', 'Windows', 'Linux'
   * @member {boolean} [vmScaleSetEnabled] Indicates if virtual machine Scale
   * Set is enabled in the specified product.
   * @member {object} [osDiskImage] OS disk image used by product.
   * @member {string} [osDiskImage.operatingSystem] OS operating system type.
   * Possible values include: 'None', 'Windows', 'Linux'
   * @member {string} [osDiskImage.sourceBlobSasUri] SAS key for source blob.
   * @member {array} [dataDiskImages] List of attached data disks.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ExtendedProduct
   *
   * @returns {object} metadata of ExtendedProduct
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ExtendedProduct',
      type: {
        name: 'Composite',
        className: 'ExtendedProduct',
        modelProperties: {
          galleryPackageBlobSasUri: {
            required: false,
            readOnly: true,
            serializedName: 'galleryPackageBlobSasUri',
            type: {
              name: 'String'
            }
          },
          productKind: {
            required: false,
            readOnly: true,
            serializedName: 'productKind',
            type: {
              name: 'String'
            }
          },
          computeRole: {
            required: false,
            readOnly: true,
            serializedName: 'properties.computeRole',
            type: {
              name: 'String'
            }
          },
          isSystemExtension: {
            required: false,
            readOnly: true,
            serializedName: 'properties.isSystemExtension',
            type: {
              name: 'Boolean'
            }
          },
          uri: {
            required: false,
            readOnly: true,
            serializedName: 'properties.sourceBlob.uri',
            type: {
              name: 'String'
            }
          },
          supportMultipleExtensions: {
            required: false,
            readOnly: true,
            serializedName: 'properties.supportMultipleExtensions',
            type: {
              name: 'Boolean'
            }
          },
          version: {
            required: false,
            readOnly: true,
            serializedName: 'properties.version',
            type: {
              name: 'String'
            }
          },
          vmOsType: {
            required: false,
            readOnly: true,
            serializedName: 'properties.vmOsType',
            type: {
              name: 'String'
            }
          },
          vmScaleSetEnabled: {
            required: false,
            readOnly: true,
            serializedName: 'properties.vmScaleSetEnabled',
            type: {
              name: 'Boolean'
            }
          },
          osDiskImage: {
            required: false,
            readOnly: true,
            serializedName: 'properties.osDiskImage',
            type: {
              name: 'Composite',
              className: 'OsDiskImage'
            }
          },
          dataDiskImages: {
            required: false,
            readOnly: true,
            serializedName: 'properties.dataDiskImages',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'DataDiskImageElementType',
                  type: {
                    name: 'Composite',
                    className: 'DataDiskImage'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ExtendedProduct;
