/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * @summary A reference to the Azure Virtual Machines Marketplace Image and
 * additional information about the Image.
  *
 */
class ImageInformation {
  /**
   * Create a ImageInformation.
   * @property {string} nodeAgentSKUId The ID of the Compute Node agent SKU
   * which the Image supports.
   * @property {object} imageReference The reference to the Azure Virtual
   * Machine's Marketplace Image.
   * @property {string} [imageReference.publisher] For example, Canonical or
   * MicrosoftWindowsServer.
   * @property {string} [imageReference.offer] For example, UbuntuServer or
   * WindowsServer.
   * @property {string} [imageReference.sku] For example, 18.04-LTS or
   * 2019-Datacenter.
   * @property {string} [imageReference.version] A value of 'latest' can be
   * specified to select the latest version of an Image. If omitted, the
   * default is 'latest'.
   * @property {string} [imageReference.virtualMachineImageId] This property is
   * mutually exclusive with other ImageReference properties. For Virtual
   * Machine Image it must be in the same region and subscription as the Azure
   * Batch account. The Shared Image Gallery Image must have replicas in the
   * same region as the Azure Batch account. For information about the firewall
   * settings for the Batch Compute Node agent to communicate with the Batch
   * service see
   * https://docs.microsoft.com/en-us/azure/batch/batch-api-basics#virtual-network-vnet-and-firewall-configuration.
   * @property {string} osType The type of operating system (e.g. Windows or
   * Linux) of the Image. Possible values include: 'linux', 'windows'
   * @property {array} [capabilities] The capabilities or features which the
   * Image supports. Not every capability of the Image is listed. Capabilities
   * in this list are considered of special interest and are generally related
   * to integration with other features in the Azure Batch service.
   * @property {date} [batchSupportEndOfLife] The time when the Azure Batch
   * service will stop accepting create Pool requests for the Image.
   * @property {string} verificationType Whether the Azure Batch service
   * actively verifies that the Image is compatible with the associated Compute
   * Node agent SKU. Possible values include: 'verified', 'unverified'
   */
  constructor() {
  }

  /**
   * Defines the metadata of ImageInformation
   *
   * @returns {object} metadata of ImageInformation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ImageInformation',
      type: {
        name: 'Composite',
        className: 'ImageInformation',
        modelProperties: {
          nodeAgentSKUId: {
            required: true,
            serializedName: 'nodeAgentSKUId',
            type: {
              name: 'String'
            }
          },
          imageReference: {
            required: true,
            serializedName: 'imageReference',
            type: {
              name: 'Composite',
              className: 'ImageReference'
            }
          },
          osType: {
            required: true,
            serializedName: 'osType',
            type: {
              name: 'Enum',
              allowedValues: [ 'linux', 'windows' ]
            }
          },
          capabilities: {
            required: false,
            serializedName: 'capabilities',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          batchSupportEndOfLife: {
            required: false,
            serializedName: 'batchSupportEndOfLife',
            type: {
              name: 'DateTime'
            }
          },
          verificationType: {
            required: true,
            serializedName: 'verificationType',
            type: {
              name: 'Enum',
              allowedValues: [ 'verified', 'unverified' ]
            }
          }
        }
      }
    };
  }
}

module.exports = ImageInformation;
