/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * The network configuration for a Pool.
 *
 */
class NetworkConfiguration {
  /**
   * Create a NetworkConfiguration.
   * @property {string} [subnetId] The ARM resource identifier of the virtual
   * network subnet which the Compute Nodes of the Pool will join. This is of
   * the form
   * /subscriptions/{subscription}/resourceGroups/{group}/providers/{provider}/virtualNetworks/{network}/subnets/{subnet}.
   * The virtual network must be in the same region and subscription as the
   * Azure Batch Account. The specified subnet should have enough free IP
   * addresses to accommodate the number of Compute Nodes in the Pool. If the
   * subnet doesn't have enough free IP addresses, the Pool will partially
   * allocate Nodes and a resize error will occur. The 'MicrosoftAzureBatch'
   * service principal must have the 'Classic Virtual Machine Contributor'
   * Role-Based Access Control (RBAC) role for the specified VNet. The
   * specified subnet must allow communication from the Azure Batch service to
   * be able to schedule Tasks on the Nodes. This can be verified by checking
   * if the specified VNet has any associated Network Security Groups (NSG). If
   * communication to the Nodes in the specified subnet is denied by an NSG,
   * then the Batch service will set the state of the Compute Nodes to
   * unusable. For Pools created with virtualMachineConfiguration only ARM
   * virtual networks ('Microsoft.Network/virtualNetworks') are supported, but
   * for Pools created with cloudServiceConfiguration both ARM and classic
   * virtual networks are supported. If the specified VNet has any associated
   * Network Security Groups (NSG), then a few reserved system ports must be
   * enabled for inbound communication. For Pools created with a virtual
   * machine configuration, enable ports 29876 and 29877, as well as port 22
   * for Linux and port 3389 for Windows. For Pools created with a cloud
   * service configuration, enable ports 10100, 20100, and 30100. Also enable
   * outbound connections to Azure Storage on port 443. For more details see:
   * https://docs.microsoft.com/en-us/azure/batch/batch-api-basics#virtual-network-vnet-and-firewall-configuration
   * @property {string} [dynamicVNetAssignmentScope] The scope of dynamic vnet
   * assignment. Possible values include: 'none', 'job'
   * @property {object} [endpointConfiguration] The configuration for endpoints
   * on Compute Nodes in the Batch Pool. Pool endpoint configuration is only
   * supported on Pools with the virtualMachineConfiguration property.
   * @property {array} [endpointConfiguration.inboundNATPools] The maximum
   * number of inbound NAT Pools per Batch Pool is 5. If the maximum number of
   * inbound NAT Pools is exceeded the request fails with HTTP status code 400.
   * @property {object} [publicIPAddressConfiguration] The Public IPAddress
   * configuration for Compute Nodes in the Batch Pool. Public IP configuration
   * property is only supported on Pools with the virtualMachineConfiguration
   * property.
   * @property {string} [publicIPAddressConfiguration.provision] The default
   * value is BatchManaged. Possible values include: 'batchManaged',
   * 'userManaged', 'noPublicIPAddresses'
   * @property {array} [publicIPAddressConfiguration.ipAddressIds] The number
   * of IPs specified here limits the maximum size of the Pool - 50 dedicated
   * nodes or 20 low-priority nodes can be allocated for each public IP. For
   * example, a pool needing 150 dedicated VMs would need at least 3 public IPs
   * specified. Each element of this collection is of the form:
   * /subscriptions/{subscription}/resourceGroups/{group}/providers/Microsoft.Network/publicIPAddresses/{ip}.
   */
  constructor() {
  }

  /**
   * Defines the metadata of NetworkConfiguration
   *
   * @returns {object} metadata of NetworkConfiguration
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'NetworkConfiguration',
      type: {
        name: 'Composite',
        className: 'NetworkConfiguration',
        modelProperties: {
          subnetId: {
            required: false,
            serializedName: 'subnetId',
            type: {
              name: 'String'
            }
          },
          dynamicVNetAssignmentScope: {
            required: false,
            serializedName: 'dynamicVNetAssignmentScope',
            type: {
              name: 'Enum',
              allowedValues: [ 'none', 'job' ]
            }
          },
          endpointConfiguration: {
            required: false,
            serializedName: 'endpointConfiguration',
            type: {
              name: 'Composite',
              className: 'PoolEndpointConfiguration'
            }
          },
          publicIPAddressConfiguration: {
            required: false,
            serializedName: 'publicIPAddressConfiguration',
            type: {
              name: 'Composite',
              className: 'PublicIPAddressConfiguration'
            }
          }
        }
      }
    };
  }
}

module.exports = NetworkConfiguration;
