/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * An pricesheet resource.
 *
 * @extends models['Resource']
 */
class PriceSheetModel extends models['Resource'] {
  /**
   * Create a PriceSheetModel.
   * @member {string} [billingPeriodId] The id of the billing period resource
   * that the usage belongs to.
   * @member {string} [meterId] The meter id
   * @member {object} [meterDetails] The details about the meter. By default
   * this is not populated, unless it's specified in $expand.
   * @member {string} [meterDetails.meterName] The name of the meter, within
   * the given meter category
   * @member {string} [meterDetails.meterCategory] The category of the meter,
   * for example, 'Cloud services', 'Networking', etc..
   * @member {string} [meterDetails.meterSubCategory] The subcategory of the
   * meter, for example, 'A6 Cloud services', 'ExpressRoute (IXP)', etc..
   * @member {string} [meterDetails.unit] The unit in which the meter
   * consumption is charged, for example, 'Hours', 'GB', etc.
   * @member {string} [meterDetails.meterLocation] The location in which the
   * Azure service is available.
   * @member {number} [meterDetails.totalIncludedQuantity] The total included
   * quantity associated with the offer.
   * @member {number} [meterDetails.pretaxStandardRate] The pretax listing
   * price.
   * @member {string} [unitOfMeasure] Unit of measure
   * @member {number} [includedQuantity] Included quality for an offer
   * @member {string} [partNumber] Part Number
   * @member {number} [unitPrice] Unit Price
   * @member {string} [currencyCode] Currency Code
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of PriceSheetModel
   *
   * @returns {object} metadata of PriceSheetModel
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'PriceSheetModel',
      type: {
        name: 'Composite',
        className: 'PriceSheetModel',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            readOnly: true,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          billingPeriodId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.billingPeriodId',
            type: {
              name: 'String'
            }
          },
          meterId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.meterId',
            type: {
              name: 'String'
            }
          },
          meterDetails: {
            required: false,
            readOnly: true,
            serializedName: 'properties.meterDetails',
            type: {
              name: 'Composite',
              className: 'MeterDetails'
            }
          },
          unitOfMeasure: {
            required: false,
            readOnly: true,
            serializedName: 'properties.unitOfMeasure',
            type: {
              name: 'String'
            }
          },
          includedQuantity: {
            required: false,
            readOnly: true,
            serializedName: 'properties.includedQuantity',
            type: {
              name: 'Number'
            }
          },
          partNumber: {
            required: false,
            readOnly: true,
            serializedName: 'properties.partNumber',
            type: {
              name: 'String'
            }
          },
          unitPrice: {
            required: false,
            readOnly: true,
            serializedName: 'properties.unitPrice',
            type: {
              name: 'Number'
            }
          },
          currencyCode: {
            required: false,
            readOnly: true,
            serializedName: 'properties.currencyCode',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = PriceSheetModel;
