/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import { BaseResource, CloudError } from "ms-rest-azure";
import * as moment from "moment";

export {

  BaseResource,
  CloudError
};

export interface ImportSourceCredentials {
  /**
   * The username to authenticate with the source registry.
  */
  username?: string;
  /**
   * The password used to authenticate with the source registry.
  */
  password: string;
}

export interface ImportSource {
  /**
   * The resource identifier of the source Azure Container Registry.
  */
  resourceId?: string;
  /**
   * The address of the source registry (e.g. 'mcr.microsoft.com').
  */
  registryUri?: string;
  /**
   * Credentials used when importing from a registry uri.
  */
  credentials?: ImportSourceCredentials;
  /**
   * Repository name of the source image.
   * Specify an image by repository ('hello-world'). This will use the 'latest' tag.
   * Specify an image by tag ('hello-world:latest').
   * Specify an image by sha256-based manifest digest ('hello-world@sha256:abc123').
  */
  sourceImage: string;
}

export interface ImportImageParameters {
  /**
   * The source of the image.
  */
  source: ImportSource;
  /**
   * List of strings of the form repo[:tag]. When tag is omitted the source will be used (or
   * 'latest' if source tag is also omitted).
  */
  targetTags?: string[];
  /**
   * List of strings of repository names to do a manifest only copy. No tag will be created.
  */
  untaggedTargetRepositories?: string[];
  /**
   * When Force, any existing target tags will be overwritten. When NoForce, any existing target
   * tags will fail the operation before any copying begins. Possible values include: 'NoForce',
   * 'Force'
  */
  mode?: string;
}

/**
 * A request to check whether a container registry name is available.
*/
export interface RegistryNameCheckRequest {
  /**
   * The name of the container registry.
  */
  name: string;
}

/**
 * The result of a request to check the availability of a container registry name.
*/
export interface RegistryNameStatus {
  /**
   * The value that indicates whether the name is available.
  */
  nameAvailable?: boolean;
  /**
   * If any, the reason that the name is not available.
  */
  reason?: string;
  /**
   * If any, the error message that provides more detail for the reason that the name is not
   * available.
  */
  message?: string;
}

/**
 * The display information for a container registry operation.
*/
export interface OperationDisplayDefinition {
  /**
   * The resource provider name: Microsoft.ContainerRegistry.
  */
  provider?: string;
  /**
   * The resource on which the operation is performed.
  */
  resource?: string;
  /**
   * The operation that users can perform.
  */
  operation?: string;
  /**
   * The description for the operation.
  */
  description?: string;
}

/**
 * The definition of Azure Monitoring metric.
*/
export interface OperationMetricSpecificationDefinition {
  /**
   * Metric name.
  */
  name?: string;
  /**
   * Metric display name.
  */
  displayName?: string;
  /**
   * Metric description.
  */
  displayDescription?: string;
  /**
   * Metric unit.
  */
  unit?: string;
  /**
   * Metric aggregation type.
  */
  aggregationType?: string;
  /**
   * Internal metric name.
  */
  internalMetricName?: string;
}

/**
 * The definition of Azure Monitoring metrics list.
*/
export interface OperationServiceSpecificationDefinition {
  /**
   * A list of Azure Monitoring metrics definition.
  */
  metricSpecifications?: OperationMetricSpecificationDefinition[];
}

/**
 * The definition of a container registry operation.
*/
export interface OperationDefinition {
  /**
   * The origin information of the container registry operation.
  */
  origin?: string;
  /**
   * Operation name: {provider}/{resource}/{operation}.
  */
  name?: string;
  /**
   * The display information for the container registry operation.
  */
  display?: OperationDisplayDefinition;
  /**
   * The definition of Azure Monitoring service.
  */
  serviceSpecification?: OperationServiceSpecificationDefinition;
}

/**
 * The SKU of a container registry.
*/
export interface Sku {
  /**
   * The SKU name of the container registry. Required for registry creation. Possible values
   * include: 'Classic', 'Basic', 'Standard', 'Premium'
  */
  name: string;
  /**
   * The SKU tier based on the SKU name. Possible values include: 'Classic', 'Basic', 'Standard',
   * 'Premium'
  */
  readonly tier?: string;
}

/**
 * The status of an Azure resource at the time the operation was called.
*/
export interface Status {
  /**
   * The short label for the status.
  */
  readonly displayStatus?: string;
  /**
   * The detailed message for the status, including alerts and error messages.
  */
  readonly message?: string;
  /**
   * The timestamp when the status was changed to the current value.
  */
  readonly timestamp?: Date;
}

/**
 * The properties of a storage account for a container registry. Only applicable to Classic SKU.
*/
export interface StorageAccountProperties {
  /**
   * The resource ID of the storage account.
  */
  id: string;
}

/**
 * Virtual network rule.
*/
export interface VirtualNetworkRule {
  /**
   * The action of virtual network rule. Possible values include: 'Allow'
  */
  action?: string;
  /**
   * Resource ID of a subnet, for example:
   * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/virtualNetworks/{vnetName}/subnets/{subnetName}.
  */
  virtualNetworkResourceId: string;
}

/**
 * IP rule with specific IP or IP range in CIDR format.
*/
export interface IPRule {
  /**
   * The action of IP ACL rule. Possible values include: 'Allow'
  */
  action?: string;
  /**
   * Specifies the IP or IP range in CIDR format. Only IPV4 address is allowed.
  */
  iPAddressOrRange: string;
}

/**
 * The network rule set for a container registry.
*/
export interface NetworkRuleSet {
  /**
   * The default action of allow or deny when no other rules match. Possible values include:
   * 'Allow', 'Deny'
  */
  defaultAction: string;
  /**
   * The virtual network rules.
  */
  virtualNetworkRules?: VirtualNetworkRule[];
  /**
   * The IP ACL rules.
  */
  ipRules?: IPRule[];
}

/**
 * An Azure resource.
*/
export interface Resource extends BaseResource {
  /**
   * The resource ID.
  */
  readonly id?: string;
  /**
   * The name of the resource.
  */
  readonly name?: string;
  /**
   * The type of the resource.
  */
  readonly type?: string;
  /**
   * The location of the resource. This cannot be changed after the resource is created.
  */
  location: string;
  /**
   * The tags of the resource.
  */
  tags?: { [propertyName: string]: string };
}

/**
 * An object that represents a container registry.
*/
export interface Registry extends Resource {
  /**
   * The SKU of the container registry.
  */
  sku: Sku;
  /**
   * The URL that can be used to log into the container registry.
  */
  readonly loginServer?: string;
  /**
   * The creation date of the container registry in ISO8601 format.
  */
  readonly creationDate?: Date;
  /**
   * The provisioning state of the container registry at the time the operation was called.
   * Possible values include: 'Creating', 'Updating', 'Deleting', 'Succeeded', 'Failed', 'Canceled'
  */
  readonly provisioningState?: string;
  /**
   * The status of the container registry at the time the operation was called.
  */
  readonly status?: Status;
  /**
   * The value that indicates whether the admin user is enabled.
  */
  adminUserEnabled?: boolean;
  /**
   * The properties of the storage account for the container registry. Only applicable to Classic
   * SKU.
  */
  storageAccount?: StorageAccountProperties;
  /**
   * The network rule set for a container registry.
  */
  networkRuleSet?: NetworkRuleSet;
}

/**
 * The parameters for updating a container registry.
*/
export interface RegistryUpdateParameters {
  /**
   * The tags for the container registry.
  */
  tags?: { [propertyName: string]: string };
  /**
   * The SKU of the container registry.
  */
  sku?: Sku;
  /**
   * The value that indicates whether the admin user is enabled.
  */
  adminUserEnabled?: boolean;
  /**
   * The parameters of a storage account for the container registry. Only applicable to Classic
   * SKU. If specified, the storage account must be in the same physical location as the container
   * registry.
  */
  storageAccount?: StorageAccountProperties;
  /**
   * The network rule set for a container registry.
  */
  networkRuleSet?: NetworkRuleSet;
}

/**
 * The login password for the container registry.
*/
export interface RegistryPassword {
  /**
   * The password name. Possible values include: 'password', 'password2'
  */
  name?: string;
  /**
   * The password value.
  */
  value?: string;
}

/**
 * The response from the ListCredentials operation.
*/
export interface RegistryListCredentialsResult {
  /**
   * The username for a container registry.
  */
  username?: string;
  /**
   * The list of passwords for a container registry.
  */
  passwords?: RegistryPassword[];
}

/**
 * The parameters used to regenerate the login credential.
*/
export interface RegenerateCredentialParameters {
  /**
   * Specifies name of the password which should be regenerated -- password or password2. Possible
   * values include: 'password', 'password2'
  */
  name: string;
}

/**
 * The quota usage for a container registry.
*/
export interface RegistryUsage {
  /**
   * The name of the usage.
  */
  name?: string;
  /**
   * The limit of the usage.
  */
  limit?: number;
  /**
   * The current value of the usage.
  */
  currentValue?: number;
  /**
   * The unit of measurement. Possible values include: 'Count', 'Bytes'
  */
  unit?: string;
}

/**
 * The result of a request to get container registry quota usages.
*/
export interface RegistryUsageListResult {
  /**
   * The list of container registry quota usages.
  */
  value?: RegistryUsage[];
}

/**
 * An object that represents quarantine policy for a container registry.
*/
export interface QuarantinePolicy {
  /**
   * The value that indicates whether the policy is enabled or not. Possible values include:
   * 'enabled', 'disabled'
  */
  status?: string;
}

/**
 * An object that represents content trust policy for a container registry.
*/
export interface TrustPolicy {
  /**
   * The type of trust policy. Possible values include: 'Notary'
  */
  type?: string;
  /**
   * The value that indicates whether the policy is enabled or not. Possible values include:
   * 'enabled', 'disabled'
  */
  status?: string;
}

/**
 * An object that represents policies for a container registry.
*/
export interface RegistryPolicies {
  /**
   * An object that represents quarantine policy for a container registry.
  */
  quarantinePolicy?: QuarantinePolicy;
  /**
   * An object that represents content trust policy for a container registry.
  */
  trustPolicy?: TrustPolicy;
}

/**
 * An object that represents a replication for a container registry.
*/
export interface Replication extends Resource {
  /**
   * The provisioning state of the replication at the time the operation was called. Possible
   * values include: 'Creating', 'Updating', 'Deleting', 'Succeeded', 'Failed', 'Canceled'
  */
  readonly provisioningState?: string;
  /**
   * The status of the replication at the time the operation was called.
  */
  readonly status?: Status;
}

/**
 * The parameters for updating a replication.
*/
export interface ReplicationUpdateParameters {
  /**
   * The tags for the replication.
  */
  tags?: { [propertyName: string]: string };
}

/**
 * An object that represents a webhook for a container registry.
*/
export interface Webhook extends Resource {
  /**
   * The status of the webhook at the time the operation was called. Possible values include:
   * 'enabled', 'disabled'
  */
  status?: string;
  /**
   * The scope of repositories where the event can be triggered. For example, 'foo:*' means events
   * for all tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only. 'foo' is
   * equivalent to 'foo:latest'. Empty means all events.
  */
  scope?: string;
  /**
   * The list of actions that trigger the webhook to post notifications.
  */
  actions: string[];
  /**
   * The provisioning state of the webhook at the time the operation was called. Possible values
   * include: 'Creating', 'Updating', 'Deleting', 'Succeeded', 'Failed', 'Canceled'
  */
  readonly provisioningState?: string;
}

/**
 * The parameters for creating a webhook.
*/
export interface WebhookCreateParameters {
  /**
   * The tags for the webhook.
  */
  tags?: { [propertyName: string]: string };
  /**
   * The location of the webhook. This cannot be changed after the resource is created.
  */
  location: string;
  /**
   * The service URI for the webhook to post notifications.
  */
  serviceUri: string;
  /**
   * Custom headers that will be added to the webhook notifications.
  */
  customHeaders?: { [propertyName: string]: string };
  /**
   * The status of the webhook at the time the operation was called. Possible values include:
   * 'enabled', 'disabled'
  */
  status?: string;
  /**
   * The scope of repositories where the event can be triggered. For example, 'foo:*' means events
   * for all tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only. 'foo' is
   * equivalent to 'foo:latest'. Empty means all events.
  */
  scope?: string;
  /**
   * The list of actions that trigger the webhook to post notifications.
  */
  actions: string[];
}

/**
 * The parameters for updating a webhook.
*/
export interface WebhookUpdateParameters {
  /**
   * The tags for the webhook.
  */
  tags?: { [propertyName: string]: string };
  /**
   * The service URI for the webhook to post notifications.
  */
  serviceUri?: string;
  /**
   * Custom headers that will be added to the webhook notifications.
  */
  customHeaders?: { [propertyName: string]: string };
  /**
   * The status of the webhook at the time the operation was called. Possible values include:
   * 'enabled', 'disabled'
  */
  status?: string;
  /**
   * The scope of repositories where the event can be triggered. For example, 'foo:*' means events
   * for all tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only. 'foo' is
   * equivalent to 'foo:latest'. Empty means all events.
  */
  scope?: string;
  /**
   * The list of actions that trigger the webhook to post notifications.
  */
  actions?: string[];
}

/**
 * The basic information of an event.
*/
export interface EventInfo {
  /**
   * The event ID.
  */
  id?: string;
}

/**
 * The configuration of service URI and custom headers for the webhook.
*/
export interface CallbackConfig {
  /**
   * The service URI for the webhook to post notifications.
  */
  serviceUri: string;
  /**
   * Custom headers that will be added to the webhook notifications.
  */
  customHeaders?: { [propertyName: string]: string };
}

/**
 * The target of the event.
*/
export interface Target {
  /**
   * The MIME type of the referenced object.
  */
  mediaType?: string;
  /**
   * The number of bytes of the content. Same as Length field.
  */
  size?: number;
  /**
   * The digest of the content, as defined by the Registry V2 HTTP API Specification.
  */
  digest?: string;
  /**
   * The number of bytes of the content. Same as Size field.
  */
  length?: number;
  /**
   * The repository name.
  */
  repository?: string;
  /**
   * The direct URL to the content.
  */
  url?: string;
  /**
   * The tag name.
  */
  tag?: string;
  /**
   * The name of the artifact.
  */
  name?: string;
  /**
   * The version of the artifact.
  */
  version?: string;
}

/**
 * The request that generated the event.
*/
export interface Request {
  /**
   * The ID of the request that initiated the event.
  */
  id?: string;
  /**
   * The IP or hostname and possibly port of the client connection that initiated the event. This
   * is the RemoteAddr from the standard http request.
  */
  addr?: string;
  /**
   * The externally accessible hostname of the registry instance, as specified by the http host
   * header on incoming requests.
  */
  host?: string;
  /**
   * The request method that generated the event.
  */
  method?: string;
  /**
   * The user agent header of the request.
  */
  useragent?: string;
}

/**
 * The agent that initiated the event. For most situations, this could be from the authorization
 * context of the request.
*/
export interface Actor {
  /**
   * The subject or username associated with the request context that generated the event.
  */
  name?: string;
}

/**
 * The registry node that generated the event. Put differently, while the actor initiates the
 * event, the source generates it.
*/
export interface Source {
  /**
   * The IP or hostname and the port of the registry node that generated the event. Generally, this
   * will be resolved by os.Hostname() along with the running port.
  */
  addr?: string;
  /**
   * The running instance of an application. Changes after each restart.
  */
  instanceID?: string;
}

/**
 * The content of the event request message.
*/
export interface EventContent {
  /**
   * The event ID.
  */
  id?: string;
  /**
   * The time at which the event occurred.
  */
  timestamp?: Date;
  /**
   * The action that encompasses the provided event.
  */
  action?: string;
  /**
   * The target of the event.
  */
  target?: Target;
  /**
   * The request that generated the event.
  */
  request?: Request;
  /**
   * The agent that initiated the event. For most situations, this could be from the authorization
   * context of the request.
  */
  actor?: Actor;
  /**
   * The registry node that generated the event. Put differently, while the actor initiates the
   * event, the source generates it.
  */
  source?: Source;
}

/**
 * The event request message sent to the service URI.
*/
export interface EventRequestMessage {
  /**
   * The content of the event request message.
  */
  content?: EventContent;
  /**
   * The headers of the event request message.
  */
  headers?: { [propertyName: string]: string };
  /**
   * The HTTP method used to send the event request message.
  */
  method?: string;
  /**
   * The URI used to send the event request message.
  */
  requestUri?: string;
  /**
   * The HTTP message version.
  */
  version?: string;
}

/**
 * The event response message received from the service URI.
*/
export interface EventResponseMessage {
  /**
   * The content of the event response message.
  */
  content?: string;
  /**
   * The headers of the event response message.
  */
  headers?: { [propertyName: string]: string };
  /**
   * The reason phrase of the event response message.
  */
  reasonPhrase?: string;
  /**
   * The status code of the event response message.
  */
  statusCode?: string;
  /**
   * The HTTP message version.
  */
  version?: string;
}

/**
 * The event for a webhook.
*/
export interface Event extends EventInfo {
  /**
   * The event request message sent to the service URI.
  */
  eventRequestMessage?: EventRequestMessage;
  /**
   * The event response message received from the service URI.
  */
  eventResponseMessage?: EventResponseMessage;
}

/**
 * The request parameters for scheduling a run.
*/
export interface RunRequest {
  /**
   * The value that indicates whether archiving is enabled for the run or not.
  */
  isArchiveEnabled?: boolean;
  /**
   * Polymorphic Discriminator
  */
  type: string;
}

/**
 * Properties for a registry image.
*/
export interface ImageDescriptor {
  /**
   * The registry login server.
  */
  registry?: string;
  /**
   * The repository name.
  */
  repository?: string;
  /**
   * The tag name.
  */
  tag?: string;
  /**
   * The sha256-based digest of the image manifest.
  */
  digest?: string;
}

/**
 * The image update trigger that caused a build.
*/
export interface ImageUpdateTrigger {
  /**
   * The unique ID of the trigger.
  */
  id?: string;
  /**
   * The timestamp when the image update happened.
  */
  timestamp?: Date;
  /**
   * The list of image updates that caused the build.
  */
  images?: ImageDescriptor[];
}

/**
 * The source trigger that caused a run.
*/
export interface SourceTriggerDescriptor {
  /**
   * The unique ID of the trigger.
  */
  id?: string;
  /**
   * The event type of the trigger.
  */
  eventType?: string;
  /**
   * The unique ID that identifies a commit.
  */
  commitId?: string;
  /**
   * The unique ID that identifies pull request.
  */
  pullRequestId?: string;
  /**
   * The repository URL.
  */
  repositoryUrl?: string;
  /**
   * The branch name in the repository.
  */
  branchName?: string;
  /**
   * The source control provider type.
  */
  providerType?: string;
}

/**
 * The platform properties against which the run has to happen.
*/
export interface PlatformProperties {
  /**
   * The operating system type required for the run. Possible values include: 'Windows', 'Linux'
  */
  os: string;
  /**
   * The OS architecture. Possible values include: 'amd64', 'x86', 'arm'
  */
  architecture?: string;
  /**
   * Variant of the CPU. Possible values include: 'v6', 'v7', 'v8'
  */
  variant?: string;
}

/**
 * The properties that determine the run agent configuration.
*/
export interface AgentProperties {
  /**
   * The CPU configuration in terms of number of cores required for the run.
  */
  cpu?: number;
}

/**
 * The resource model definition for a ARM proxy resource. It will have everything other than
 * required location and tags.
*/
export interface ProxyResource extends BaseResource {
  /**
   * The resource ID.
  */
  readonly id?: string;
  /**
   * The name of the resource.
  */
  readonly name?: string;
  /**
   * The type of the resource.
  */
  readonly type?: string;
}

/**
 * Run resource properties
*/
export interface Run extends ProxyResource {
  /**
   * The unique identifier for the run.
  */
  runId?: string;
  /**
   * The current status of the run. Possible values include: 'Queued', 'Started', 'Running',
   * 'Succeeded', 'Failed', 'Canceled', 'Error', 'Timeout'
  */
  status?: string;
  /**
   * The last updated time for the run.
  */
  lastUpdatedTime?: Date;
  /**
   * The type of run. Possible values include: 'QuickBuild', 'QuickRun', 'AutoBuild', 'AutoRun'
  */
  runType?: string;
  /**
   * The time the run was scheduled.
  */
  createTime?: Date;
  /**
   * The time the run started.
  */
  startTime?: Date;
  /**
   * The time the run finished.
  */
  finishTime?: Date;
  /**
   * The list of all images that were generated from the run. This is applicable if the run
   * generates base image dependencies.
  */
  outputImages?: ImageDescriptor[];
  /**
   * The task against which run was scheduled.
  */
  task?: string;
  /**
   * The image update trigger that caused the run. This is applicable if the task has base image
   * trigger configured.
  */
  imageUpdateTrigger?: ImageUpdateTrigger;
  /**
   * The source trigger that caused the run.
  */
  sourceTrigger?: SourceTriggerDescriptor;
  /**
   * The platform properties against which the run will happen.
  */
  platform?: PlatformProperties;
  /**
   * The machine configuration of the run agent.
  */
  agentConfiguration?: AgentProperties;
  /**
   * The scope of the credentials that were used to login to the source registry during this run.
  */
  sourceRegistryAuth?: string;
  /**
   * The list of custom registries that were logged in during this run.
  */
  customRegistries?: string[];
  /**
   * The error message received from backend systems after the run is scheduled.
  */
  readonly runErrorMessage?: string;
  /**
   * The provisioning state of a run. Possible values include: 'Creating', 'Updating', 'Deleting',
   * 'Succeeded', 'Failed', 'Canceled'
  */
  provisioningState?: string;
  /**
   * The value that indicates whether archiving is enabled or not.
  */
  isArchiveEnabled?: boolean;
}

/**
 * The properties of a response to source upload request.
*/
export interface SourceUploadDefinition {
  /**
   * The URL where the client can upload the source.
  */
  uploadUrl?: string;
  /**
   * The relative path to the source. This is used to submit the subsequent queue build request.
  */
  relativePath?: string;
}

/**
 * Properties that are enabled for Odata querying on runs.
*/
export interface RunFilter {
  /**
   * The unique identifier for the run.
  */
  runId?: string;
  /**
   * The type of run. Possible values include: 'QuickBuild', 'QuickRun', 'AutoBuild', 'AutoRun'
  */
  runType?: string;
  /**
   * The current status of the run. Possible values include: 'Queued', 'Started', 'Running',
   * 'Succeeded', 'Failed', 'Canceled', 'Error', 'Timeout'
  */
  status?: string;
  /**
   * The create time for a run.
  */
  createTime?: Date;
  /**
   * The time the run finished.
  */
  finishTime?: Date;
  /**
   * The list of comma-separated image manifests that were generated from the run. This is
   * applicable if the run is of
   * build type.
  */
  outputImageManifests?: string;
  /**
   * The value that indicates whether archiving is enabled or not.
  */
  isArchiveEnabled?: boolean;
  /**
   * The name of the task that the run corresponds to.
  */
  taskName?: string;
}

/**
 * The set of run properties that can be updated.
*/
export interface RunUpdateParameters {
  /**
   * The value that indicates whether archiving is enabled or not.
  */
  isArchiveEnabled?: boolean;
}

/**
 * The result of get log link operation.
*/
export interface RunGetLogResult {
  /**
   * The link to logs for a run on a azure container registry.
  */
  logLink?: string;
}

export interface UserIdentityProperties {
  /**
   * The principal id of user assigned identity.
  */
  principalId?: string;
  /**
   * The client id of user assigned identity.
  */
  clientId?: string;
}

/**
 * Managed identity for the resource.
*/
export interface IdentityProperties {
  /**
   * The principal ID of resource identity.
  */
  principalId?: string;
  /**
   * The tenant ID of resource.
  */
  tenantId?: string;
  /**
   * The identity type. Possible values include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned,
   * UserAssigned', 'None'
  */
  type?: string;
  /**
   * The list of user identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
  */
  userAssignedIdentities?: { [propertyName: string]: UserIdentityProperties };
}

/**
 * Properties that describe a base image dependency.
*/
export interface BaseImageDependency {
  /**
   * The type of the base image dependency. Possible values include: 'BuildTime', 'RunTime'
  */
  type?: string;
  /**
   * The registry login server.
  */
  registry?: string;
  /**
   * The repository name.
  */
  repository?: string;
  /**
   * The tag name.
  */
  tag?: string;
  /**
   * The sha256-based digest of the image manifest.
  */
  digest?: string;
}

/**
 * Base properties for any task step.
*/
export interface TaskStepProperties {
  /**
   * List of base image dependencies for a step.
  */
  readonly baseImageDependencies?: BaseImageDependency[];
  /**
   * The URL(absolute or relative) of the source context for the task step.
  */
  contextPath?: string;
  /**
   * The token (git PAT or SAS token of storage account blob) associated with the context for a
   * step.
  */
  contextAccessToken?: string;
  /**
   * Polymorphic Discriminator
  */
  type: string;
}

/**
 * The properties of a timer trigger.
*/
export interface TimerTrigger {
  /**
   * The CRON expression for the task schedule
  */
  schedule: string;
  /**
   * The current status of trigger. Possible values include: 'Disabled', 'Enabled'
  */
  status?: string;
  /**
   * The name of the trigger.
  */
  name: string;
}

/**
 * The authorization properties for accessing the source code repository.
*/
export interface AuthInfo {
  /**
   * The type of Auth token. Possible values include: 'PAT', 'OAuth'
  */
  tokenType: string;
  /**
   * The access token used to access the source control provider.
  */
  token: string;
  /**
   * The refresh token used to refresh the access token.
  */
  refreshToken?: string;
  /**
   * The scope of the access token.
  */
  scope?: string;
  /**
   * Time in seconds that the token remains valid
  */
  expiresIn?: number;
}

/**
 * The properties of the source code repository.
*/
export interface SourceProperties {
  /**
   * The type of source control service. Possible values include: 'Github',
   * 'VisualStudioTeamService'
  */
  sourceControlType: string;
  /**
   * The full URL to the source code repository
  */
  repositoryUrl: string;
  /**
   * The branch name of the source code.
  */
  branch?: string;
  /**
   * The authorization properties for accessing the source code repository and to set up
   * webhooks for notifications.
  */
  sourceControlAuthProperties?: AuthInfo;
}

/**
 * The properties of a source based trigger.
*/
export interface SourceTrigger {
  /**
   * The properties that describes the source(code) for the task.
  */
  sourceRepository: SourceProperties;
  /**
   * The source event corresponding to the trigger.
  */
  sourceTriggerEvents: string[];
  /**
   * The current status of trigger. Possible values include: 'Disabled', 'Enabled'
  */
  status?: string;
  /**
   * The name of the trigger.
  */
  name: string;
}

/**
 * The trigger based on base image dependency.
*/
export interface BaseImageTrigger {
  /**
   * The type of the auto trigger for base image dependency updates. Possible values include:
   * 'All', 'Runtime'
  */
  baseImageTriggerType: string;
  /**
   * The current status of trigger. Possible values include: 'Disabled', 'Enabled'
  */
  status?: string;
  /**
   * The name of the trigger.
  */
  name: string;
}

/**
 * The properties of a trigger.
*/
export interface TriggerProperties {
  /**
   * The collection of timer triggers.
  */
  timerTriggers?: TimerTrigger[];
  /**
   * The collection of triggers based on source code repository.
  */
  sourceTriggers?: SourceTrigger[];
  /**
   * The trigger based on base image dependencies.
  */
  baseImageTrigger?: BaseImageTrigger;
}

/**
 * Describes the credential parameters for accessing the source registry.
*/
export interface SourceRegistryCredentials {
  /**
   * The authentication mode which determines the source registry login scope. The credentials for
   * the source registry
   * will be generated using the given scope. These credentials will be used to login to
   * the source registry during the run. Possible values include: 'None', 'Default'
  */
  loginMode?: string;
}

/**
 * Describes the properties of a secret object value.
*/
export interface SecretObject {
  /**
   * The value of the secret. The format of this value will be determined
   * based on the type of the secret object. If the type is Opaque, the value will be
   * used as is without any modification.
  */
  value?: string;
  /**
   * The type of the secret object which determines how the value of the secret object has to be
   * interpreted. Possible values include: 'Opaque', 'Vaultsecret'
  */
  type?: string;
}

/**
 * Describes the credentials that will be used to access a custom registry during a run.
*/
export interface CustomRegistryCredentials {
  /**
   * The username for logging into the custom registry.
  */
  userName?: SecretObject;
  /**
   * The password for logging into the custom registry. The password is a secret
   * object that allows multiple ways of providing the value for it.
  */
  password?: SecretObject;
  /**
   * Indicates the managed identity assigned to the custom credential. If a user-assigned identity
   * this value is the Client ID. If a system-assigned identity, the value will be `system`. In
   * the case of a system-assigned identity, the Client ID will be determined by the runner. This
   * identity may be used to authenticate to key vault to retrieve credentials or it may be the
   * only
   * source of authentication used for accessing the registry.
  */
  identity?: string;
}

/**
 * The parameters that describes a set of credentials that will be used when a run is invoked.
*/
export interface Credentials {
  /**
   * Describes the credential parameters for accessing the source registry.
  */
  sourceRegistry?: SourceRegistryCredentials;
  /**
   * Describes the credential parameters for accessing other custom registries. The key
   * for the dictionary item will be the registry login server (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the registry.
  */
  customRegistries?: { [propertyName: string]: CustomRegistryCredentials };
}

/**
 * The task that has the ARM resource and task properties.
 * The task will have all information to schedule a run against it.
*/
export interface Task extends Resource {
  /**
   * Identity for the resource.
  */
  identity?: IdentityProperties;
  /**
   * The provisioning state of the task. Possible values include: 'Creating', 'Updating',
   * 'Deleting', 'Succeeded', 'Failed', 'Canceled'
  */
  readonly provisioningState?: string;
  /**
   * The creation date of task.
  */
  readonly creationDate?: Date;
  /**
   * The current status of task. Possible values include: 'Disabled', 'Enabled'
  */
  status?: string;
  /**
   * The platform properties against which the run has to happen.
  */
  platform: PlatformProperties;
  /**
   * The machine configuration of the run agent.
  */
  agentConfiguration?: AgentProperties;
  /**
   * Run timeout in seconds.
  */
  timeout?: number;
  /**
   * The properties of a task step.
  */
  step: TaskStepProperties;
  /**
   * The properties that describe all triggers for the task.
  */
  trigger?: TriggerProperties;
  /**
   * The properties that describes a set of credentials that will be used when this run is invoked.
  */
  credentials?: Credentials;
}

/**
 * The properties for updating the platform configuration.
*/
export interface PlatformUpdateParameters {
  /**
   * The operating system type required for the run. Possible values include: 'Windows', 'Linux'
  */
  os?: string;
  /**
   * The OS architecture. Possible values include: 'amd64', 'x86', 'arm'
  */
  architecture?: string;
  /**
   * Variant of the CPU. Possible values include: 'v6', 'v7', 'v8'
  */
  variant?: string;
}

/**
 * Base properties for updating any task step.
*/
export interface TaskStepUpdateParameters {
  /**
   * The URL(absolute or relative) of the source context for the task step.
  */
  contextPath?: string;
  /**
   * The token (git PAT or SAS token of storage account blob) associated with the context for a
   * step.
  */
  contextAccessToken?: string;
  /**
   * Polymorphic Discriminator
  */
  type: string;
}

/**
 * The properties for updating a timer trigger.
*/
export interface TimerTriggerUpdateParameters {
  /**
   * The CRON expression for the task schedule
  */
  schedule?: string;
  /**
   * The current status of trigger. Possible values include: 'Disabled', 'Enabled'
  */
  status?: string;
  /**
   * The name of the trigger.
  */
  name: string;
}

/**
 * The authorization properties for accessing the source code repository.
*/
export interface AuthInfoUpdateParameters {
  /**
   * The type of Auth token. Possible values include: 'PAT', 'OAuth'
  */
  tokenType?: string;
  /**
   * The access token used to access the source control provider.
  */
  token?: string;
  /**
   * The refresh token used to refresh the access token.
  */
  refreshToken?: string;
  /**
   * The scope of the access token.
  */
  scope?: string;
  /**
   * Time in seconds that the token remains valid
  */
  expiresIn?: number;
}

/**
 * The properties for updating the source code repository.
*/
export interface SourceUpdateParameters {
  /**
   * The type of source control service. Possible values include: 'Github',
   * 'VisualStudioTeamService'
  */
  sourceControlType?: string;
  /**
   * The full URL to the source code repository
  */
  repositoryUrl?: string;
  /**
   * The branch name of the source code.
  */
  branch?: string;
  /**
   * The authorization properties for accessing the source code repository and to set up
   * webhooks for notifications.
  */
  sourceControlAuthProperties?: AuthInfoUpdateParameters;
}

/**
 * The properties for updating a source based trigger.
*/
export interface SourceTriggerUpdateParameters {
  /**
   * The properties that describes the source(code) for the task.
  */
  sourceRepository?: SourceUpdateParameters;
  /**
   * The source event corresponding to the trigger.
  */
  sourceTriggerEvents?: string[];
  /**
   * The current status of trigger. Possible values include: 'Disabled', 'Enabled'
  */
  status?: string;
  /**
   * The name of the trigger.
  */
  name: string;
}

/**
 * The properties for updating base image dependency trigger.
*/
export interface BaseImageTriggerUpdateParameters {
  /**
   * The type of the auto trigger for base image dependency updates. Possible values include:
   * 'All', 'Runtime'
  */
  baseImageTriggerType?: string;
  /**
   * The current status of trigger. Possible values include: 'Disabled', 'Enabled'
  */
  status?: string;
  /**
   * The name of the trigger.
  */
  name: string;
}

/**
 * The properties for updating triggers.
*/
export interface TriggerUpdateParameters {
  /**
   * The collection of timer triggers.
  */
  timerTriggers?: TimerTriggerUpdateParameters[];
  /**
   * The collection of triggers based on source code repository.
  */
  sourceTriggers?: SourceTriggerUpdateParameters[];
  /**
   * The trigger based on base image dependencies.
  */
  baseImageTrigger?: BaseImageTriggerUpdateParameters;
}

/**
 * The parameters for updating a task.
*/
export interface TaskUpdateParameters {
  /**
   * Identity for the resource.
  */
  identity?: IdentityProperties;
  /**
   * The current status of task. Possible values include: 'Disabled', 'Enabled'
  */
  status?: string;
  /**
   * The platform properties against which the run has to happen.
  */
  platform?: PlatformUpdateParameters;
  /**
   * The machine configuration of the run agent.
  */
  agentConfiguration?: AgentProperties;
  /**
   * Run timeout in seconds.
  */
  timeout?: number;
  /**
   * The properties for updating a task step.
  */
  step?: TaskStepUpdateParameters;
  /**
   * The properties for updating trigger properties.
  */
  trigger?: TriggerUpdateParameters;
  /**
   * The parameters that describes a set of credentials that will be used when this run is invoked.
  */
  credentials?: Credentials;
  /**
   * The ARM resource tags.
  */
  tags?: { [propertyName: string]: string };
}

/**
 * The properties of a run argument.
*/
export interface Argument {
  /**
   * The name of the argument.
  */
  name: string;
  /**
   * The value of the argument.
  */
  value: string;
  /**
   * Flag to indicate whether the argument represents a secret and want to be removed from build
   * logs.
  */
  isSecret?: boolean;
}

/**
 * The parameters for a docker quick build.
*/
export interface DockerBuildRequest extends RunRequest {
  /**
   * The fully qualified image names including the repository and tag.
  */
  imageNames?: string[];
  /**
   * The value of this property indicates whether the image built should be pushed to the registry
   * or not.
  */
  isPushEnabled?: boolean;
  /**
   * The value of this property indicates whether the image cache is enabled or not.
  */
  noCache?: boolean;
  /**
   * The Docker file path relative to the source location.
  */
  dockerFilePath: string;
  /**
   * The name of the target build stage for the docker build.
  */
  target?: string;
  /**
   * The collection of override arguments to be used when executing the run.
  */
  argumentsProperty?: Argument[];
  /**
   * Run timeout in seconds.
  */
  timeout?: number;
  /**
   * The platform properties against which the run has to happen.
  */
  platform: PlatformProperties;
  /**
   * The machine configuration of the run agent.
  */
  agentConfiguration?: AgentProperties;
  /**
   * The URL(absolute or relative) of the source context. It can be an URL to a tar or git
   * repository.
   * If it is relative URL, the relative path should be obtained from calling
   * listBuildSourceUploadUrl API.
  */
  sourceLocation?: string;
  /**
   * The properties that describes a set of credentials that will be used when this run is invoked.
  */
  credentials?: Credentials;
}

/**
 * The properties of a overridable value that can be passed to a task template.
*/
export interface SetValue {
  /**
   * The name of the overridable value.
  */
  name: string;
  /**
   * The overridable value.
  */
  value: string;
  /**
   * Flag to indicate whether the value represents a secret or not.
  */
  isSecret?: boolean;
}

/**
 * The request parameters for a scheduling run against a task file.
*/
export interface FileTaskRunRequest extends RunRequest {
  /**
   * The template/definition file path relative to the source.
  */
  taskFilePath: string;
  /**
   * The values/parameters file path relative to the source.
  */
  valuesFilePath?: string;
  /**
   * The collection of overridable values that can be passed when running a task.
  */
  values?: SetValue[];
  /**
   * Run timeout in seconds.
  */
  timeout?: number;
  /**
   * The platform properties against which the run has to happen.
  */
  platform: PlatformProperties;
  /**
   * The machine configuration of the run agent.
  */
  agentConfiguration?: AgentProperties;
  /**
   * The URL(absolute or relative) of the source context. It can be an URL to a tar or git
   * repository.
   * If it is relative URL, the relative path should be obtained from calling
   * listBuildSourceUploadUrl API.
  */
  sourceLocation?: string;
  /**
   * The properties that describes a set of credentials that will be used when this run is invoked.
  */
  credentials?: Credentials;
}

/**
 * The parameters for a task run request.
*/
export interface TaskRunRequest extends RunRequest {
  /**
   * The name of task against which run has to be queued.
  */
  taskName: string;
  /**
   * The collection of overridable values that can be passed when running a task.
  */
  values?: SetValue[];
}

/**
 * The parameters for a quick task run request.
*/
export interface EncodedTaskRunRequest extends RunRequest {
  /**
   * Base64 encoded value of the template/definition file content.
  */
  encodedTaskContent: string;
  /**
   * Base64 encoded value of the parameters/values file content.
  */
  encodedValuesContent?: string;
  /**
   * The collection of overridable values that can be passed when running a task.
  */
  values?: SetValue[];
  /**
   * Run timeout in seconds.
  */
  timeout?: number;
  /**
   * The platform properties against which the run has to happen.
  */
  platform: PlatformProperties;
  /**
   * The machine configuration of the run agent.
  */
  agentConfiguration?: AgentProperties;
  /**
   * The URL(absolute or relative) of the source context. It can be an URL to a tar or git
   * repository.
   * If it is relative URL, the relative path should be obtained from calling
   * listBuildSourceUploadUrl API.
  */
  sourceLocation?: string;
  /**
   * The properties that describes a set of credentials that will be used when this run is invoked.
  */
  credentials?: Credentials;
}

/**
 * The Docker build step.
*/
export interface DockerBuildStep extends TaskStepProperties {
  /**
   * The fully qualified image names including the repository and tag.
  */
  imageNames?: string[];
  /**
   * The value of this property indicates whether the image built should be pushed to the registry
   * or not.
  */
  isPushEnabled?: boolean;
  /**
   * The value of this property indicates whether the image cache is enabled or not.
  */
  noCache?: boolean;
  /**
   * The Docker file path relative to the source context.
  */
  dockerFilePath: string;
  /**
   * The name of the target build stage for the docker build.
  */
  target?: string;
  /**
   * The collection of override arguments to be used when executing this build step.
  */
  argumentsProperty?: Argument[];
}

/**
 * The properties of a task step.
*/
export interface FileTaskStep extends TaskStepProperties {
  /**
   * The task template/definition file path relative to the source context.
  */
  taskFilePath: string;
  /**
   * The task values/parameters file path relative to the source context.
  */
  valuesFilePath?: string;
  /**
   * The collection of overridable values that can be passed when running a task.
  */
  values?: SetValue[];
}

/**
 * The properties of a encoded task step.
*/
export interface EncodedTaskStep extends TaskStepProperties {
  /**
   * Base64 encoded value of the template/definition file content.
  */
  encodedTaskContent: string;
  /**
   * Base64 encoded value of the parameters/values file content.
  */
  encodedValuesContent?: string;
  /**
   * The collection of overridable values that can be passed when running a task.
  */
  values?: SetValue[];
}

/**
 * The properties for updating a docker build step.
*/
export interface DockerBuildStepUpdateParameters extends TaskStepUpdateParameters {
  /**
   * The fully qualified image names including the repository and tag.
  */
  imageNames?: string[];
  /**
   * The value of this property indicates whether the image built should be pushed to the registry
   * or not.
  */
  isPushEnabled?: boolean;
  /**
   * The value of this property indicates whether the image cache is enabled or not.
  */
  noCache?: boolean;
  /**
   * The Docker file path relative to the source context.
  */
  dockerFilePath?: string;
  /**
   * The collection of override arguments to be used when executing this build step.
  */
  argumentsProperty?: Argument[];
  /**
   * The name of the target build stage for the docker build.
  */
  target?: string;
}

/**
 * The properties of updating a task step.
*/
export interface FileTaskStepUpdateParameters extends TaskStepUpdateParameters {
  /**
   * The task template/definition file path relative to the source context.
  */
  taskFilePath?: string;
  /**
   * The values/parameters file path relative to the source context.
  */
  valuesFilePath?: string;
  /**
   * The collection of overridable values that can be passed when running a task.
  */
  values?: SetValue[];
}

/**
 * The properties for updating encoded task step.
*/
export interface EncodedTaskStepUpdateParameters extends TaskStepUpdateParameters {
  /**
   * Base64 encoded value of the template/definition file content.
  */
  encodedTaskContent?: string;
  /**
   * Base64 encoded value of the parameters/values file content.
  */
  encodedValuesContent?: string;
  /**
   * The collection of overridable values that can be passed when running a task.
  */
  values?: SetValue[];
}

/**
 * The result of a request to list container registries.
*/
export interface RegistryListResult extends Array<Registry> {
  /**
   * The URI that can be used to request the next list of container registries.
  */
  nextLink?: string;
}

/**
 * The result of a request to list container registry operations.
*/
export interface OperationListResult extends Array<OperationDefinition> {
  /**
   * The URI that can be used to request the next list of container registry operations.
  */
  nextLink?: string;
}

/**
 * The result of a request to list replications for a container registry.
*/
export interface ReplicationListResult extends Array<Replication> {
  /**
   * The URI that can be used to request the next list of replications.
  */
  nextLink?: string;
}

/**
 * The result of a request to list webhooks for a container registry.
*/
export interface WebhookListResult extends Array<Webhook> {
  /**
   * The URI that can be used to request the next list of webhooks.
  */
  nextLink?: string;
}

/**
 * The result of a request to list events for a webhook.
*/
export interface EventListResult extends Array<Event> {
  /**
   * The URI that can be used to request the next list of events.
  */
  nextLink?: string;
}

/**
 * Collection of runs.
*/
export interface RunListResult extends Array<Run> {
  /**
   * The URI that can be used to request the next set of paged results.
  */
  nextLink?: string;
}

/**
 * The collection of tasks.
*/
export interface TaskListResult extends Array<Task> {
  /**
   * The URI that can be used to request the next set of paged results.
  */
  nextLink?: string;
}
