/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The task that has the ARM resource and task properties.
 * The task will have all information to schedule a run against it.
 *
 * @extends models['Resource']
 */
class Task extends models['Resource'] {
  /**
   * Create a Task.
   * @property {object} [identity] Identity for the resource.
   * @property {string} [identity.principalId] The principal ID of resource
   * identity.
   * @property {string} [identity.tenantId] The tenant ID of resource.
   * @property {string} [identity.type] The identity type. Possible values
   * include: 'SystemAssigned', 'UserAssigned', 'SystemAssigned, UserAssigned',
   * 'None'
   * @property {object} [identity.userAssignedIdentities] The list of user
   * identities associated with the resource. The user identity
   * dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
   * providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   * @property {string} [provisioningState] The provisioning state of the task.
   * Possible values include: 'Creating', 'Updating', 'Deleting', 'Succeeded',
   * 'Failed', 'Canceled'
   * @property {date} [creationDate] The creation date of task.
   * @property {string} [status] The current status of task. Possible values
   * include: 'Disabled', 'Enabled'
   * @property {object} platform The platform properties against which the run
   * has to happen.
   * @property {string} [platform.os] The operating system type required for
   * the run. Possible values include: 'Windows', 'Linux'
   * @property {string} [platform.architecture] The OS architecture. Possible
   * values include: 'amd64', 'x86', 'arm'
   * @property {string} [platform.variant] Variant of the CPU. Possible values
   * include: 'v6', 'v7', 'v8'
   * @property {object} [agentConfiguration] The machine configuration of the
   * run agent.
   * @property {number} [agentConfiguration.cpu] The CPU configuration in terms
   * of number of cores required for the run.
   * @property {number} [timeout] Run timeout in seconds. Default value: 3600 .
   * @property {object} step The properties of a task step.
   * @property {array} [step.baseImageDependencies] List of base image
   * dependencies for a step.
   * @property {string} [step.contextPath] The URL(absolute or relative) of the
   * source context for the task step.
   * @property {string} [step.contextAccessToken] The token (git PAT or SAS
   * token of storage account blob) associated with the context for a step.
   * @property {string} [step.type] Polymorphic Discriminator
   * @property {object} [trigger] The properties that describe all triggers for
   * the task.
   * @property {array} [trigger.timerTriggers] The collection of timer
   * triggers.
   * @property {array} [trigger.sourceTriggers] The collection of triggers
   * based on source code repository.
   * @property {object} [trigger.baseImageTrigger] The trigger based on base
   * image dependencies.
   * @property {string} [trigger.baseImageTrigger.baseImageTriggerType] The
   * type of the auto trigger for base image dependency updates. Possible
   * values include: 'All', 'Runtime'
   * @property {string} [trigger.baseImageTrigger.status] The current status of
   * trigger. Possible values include: 'Disabled', 'Enabled'
   * @property {string} [trigger.baseImageTrigger.name] The name of the
   * trigger.
   * @property {object} [credentials] The properties that describes a set of
   * credentials that will be used when this run is invoked.
   * @property {object} [credentials.sourceRegistry] Describes the credential
   * parameters for accessing the source registry.
   * @property {string} [credentials.sourceRegistry.loginMode] The
   * authentication mode which determines the source registry login scope. The
   * credentials for the source registry
   * will be generated using the given scope. These credentials will be used to
   * login to
   * the source registry during the run. Possible values include: 'None',
   * 'Default'
   * @property {object} [credentials.customRegistries] Describes the credential
   * parameters for accessing other custom registries. The key
   * for the dictionary item will be the registry login server
   * (myregistry.azurecr.io) and
   * the value of the item will be the registry credentials for accessing the
   * registry.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Task
   *
   * @returns {object} metadata of Task
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Task',
      type: {
        name: 'Composite',
        className: 'Task',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          identity: {
            required: false,
            serializedName: 'identity',
            type: {
              name: 'Composite',
              className: 'IdentityProperties'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          creationDate: {
            required: false,
            readOnly: true,
            serializedName: 'properties.creationDate',
            type: {
              name: 'DateTime'
            }
          },
          status: {
            required: false,
            serializedName: 'properties.status',
            type: {
              name: 'String'
            }
          },
          platform: {
            required: true,
            serializedName: 'properties.platform',
            type: {
              name: 'Composite',
              className: 'PlatformProperties'
            }
          },
          agentConfiguration: {
            required: false,
            serializedName: 'properties.agentConfiguration',
            type: {
              name: 'Composite',
              className: 'AgentProperties'
            }
          },
          timeout: {
            required: false,
            serializedName: 'properties.timeout',
            defaultValue: 3600,
            constraints: {
              InclusiveMaximum: 28800,
              InclusiveMinimum: 300
            },
            type: {
              name: 'Number'
            }
          },
          step: {
            required: true,
            serializedName: 'properties.step',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'TaskStepProperties',
              className: 'TaskStepProperties'
            }
          },
          trigger: {
            required: false,
            serializedName: 'properties.trigger',
            type: {
              name: 'Composite',
              className: 'TriggerProperties'
            }
          },
          credentials: {
            required: false,
            serializedName: 'properties.credentials',
            type: {
              name: 'Composite',
              className: 'Credentials'
            }
          }
        }
      }
    };
  }
}

module.exports = Task;
