/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { ServiceClient, ServiceClientOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from "./models";

export default class PredictionAPIClient extends ServiceClient {
  /**
   * @class
   * Initializes a new instance of the PredictionAPIClient class.
   * @constructor
   *
   * @param {string} apiKey - API key.
   *
   * @param {string} endpoint - Supported Cognitive Services endpoints.
   *
   * @param {object} [options] - The parameter options
   *
   * @param {Array} [options.filters] - Filters to be added to the request pipeline
   *
   * @param {object} [options.requestOptions] - Options for the underlying request object
   * {@link https://github.com/request/request#requestoptions-callback Options doc}
   *
   * @param {boolean} [options.noRetryPolicy] - If set to true, turn off default retry policy
   *
   */
  constructor(apiKey: string, endpoint: string, options?: ServiceClientOptions);

  apiKey: string;

  endpoint: string;


  /**
   * @summary Classify an image url and saves the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageUrl An ImageUrl that contains the url of the image to
   * be evaluated.
   *
   * @param {string} imageUrl.url Url of the image.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ImagePrediction>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  classifyImageUrlWithHttpOperationResponse(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ImagePrediction>>;

  /**
   * @summary Classify an image url and saves the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageUrl An ImageUrl that contains the url of the image to
   * be evaluated.
   *
   * @param {string} imageUrl.url Url of the image.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ImagePrediction} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ImagePrediction} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ImagePrediction} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  classifyImageUrl(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ImagePrediction>;
  classifyImageUrl(projectId: string, publishedName: string, imageUrl: models.ImageUrl, callback: ServiceCallback<models.ImagePrediction>): void;
  classifyImageUrl(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options: { application? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ImagePrediction>): void;


  /**
   * @summary Classify an image and saves the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageData Binary image data. Supported formats are JPEG,
   * GIF, PNG, and BMP. Supports images up to 4MB.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ImagePrediction>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  classifyImageWithHttpOperationResponse(projectId: string, publishedName: string, imageData: stream.Readable, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ImagePrediction>>;

  /**
   * @summary Classify an image and saves the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageData Binary image data. Supported formats are JPEG,
   * GIF, PNG, and BMP. Supports images up to 4MB.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ImagePrediction} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ImagePrediction} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ImagePrediction} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  classifyImage(projectId: string, publishedName: string, imageData: stream.Readable, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ImagePrediction>;
  classifyImage(projectId: string, publishedName: string, imageData: stream.Readable, callback: ServiceCallback<models.ImagePrediction>): void;
  classifyImage(projectId: string, publishedName: string, imageData: stream.Readable, options: { application? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ImagePrediction>): void;


  /**
   * @summary Classify an image url without saving the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageUrl An {Iris.Web.Api.Models.ImageUrl} that contains the
   * url of the image to be evaluated.
   *
   * @param {string} imageUrl.url Url of the image.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ImagePrediction>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  classifyImageUrlWithNoStoreWithHttpOperationResponse(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ImagePrediction>>;

  /**
   * @summary Classify an image url without saving the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageUrl An {Iris.Web.Api.Models.ImageUrl} that contains the
   * url of the image to be evaluated.
   *
   * @param {string} imageUrl.url Url of the image.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ImagePrediction} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ImagePrediction} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ImagePrediction} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  classifyImageUrlWithNoStore(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ImagePrediction>;
  classifyImageUrlWithNoStore(projectId: string, publishedName: string, imageUrl: models.ImageUrl, callback: ServiceCallback<models.ImagePrediction>): void;
  classifyImageUrlWithNoStore(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options: { application? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ImagePrediction>): void;


  /**
   * @summary Classify an image without saving the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageData Binary image data. Supported formats are JPEG,
   * GIF, PNG, and BMP. Supports images up to 0MB.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ImagePrediction>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  classifyImageWithNoStoreWithHttpOperationResponse(projectId: string, publishedName: string, imageData: stream.Readable, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ImagePrediction>>;

  /**
   * @summary Classify an image without saving the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageData Binary image data. Supported formats are JPEG,
   * GIF, PNG, and BMP. Supports images up to 0MB.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ImagePrediction} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ImagePrediction} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ImagePrediction} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  classifyImageWithNoStore(projectId: string, publishedName: string, imageData: stream.Readable, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ImagePrediction>;
  classifyImageWithNoStore(projectId: string, publishedName: string, imageData: stream.Readable, callback: ServiceCallback<models.ImagePrediction>): void;
  classifyImageWithNoStore(projectId: string, publishedName: string, imageData: stream.Readable, options: { application? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ImagePrediction>): void;


  /**
   * @summary Detect objects in an image url and saves the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageUrl An ImageUrl that contains the url of the image to
   * be evaluated.
   *
   * @param {string} imageUrl.url Url of the image.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ImagePrediction>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  detectImageUrlWithHttpOperationResponse(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ImagePrediction>>;

  /**
   * @summary Detect objects in an image url and saves the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageUrl An ImageUrl that contains the url of the image to
   * be evaluated.
   *
   * @param {string} imageUrl.url Url of the image.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ImagePrediction} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ImagePrediction} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ImagePrediction} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  detectImageUrl(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ImagePrediction>;
  detectImageUrl(projectId: string, publishedName: string, imageUrl: models.ImageUrl, callback: ServiceCallback<models.ImagePrediction>): void;
  detectImageUrl(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options: { application? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ImagePrediction>): void;


  /**
   * @summary Detect objects in an image and saves the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageData Binary image data. Supported formats are JPEG,
   * GIF, PNG, and BMP. Supports images up to 4MB.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ImagePrediction>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  detectImageWithHttpOperationResponse(projectId: string, publishedName: string, imageData: stream.Readable, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ImagePrediction>>;

  /**
   * @summary Detect objects in an image and saves the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageData Binary image data. Supported formats are JPEG,
   * GIF, PNG, and BMP. Supports images up to 4MB.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ImagePrediction} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ImagePrediction} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ImagePrediction} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  detectImage(projectId: string, publishedName: string, imageData: stream.Readable, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ImagePrediction>;
  detectImage(projectId: string, publishedName: string, imageData: stream.Readable, callback: ServiceCallback<models.ImagePrediction>): void;
  detectImage(projectId: string, publishedName: string, imageData: stream.Readable, options: { application? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ImagePrediction>): void;


  /**
   * @summary Detect objects in an image url without saving the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageUrl An {Iris.Web.Api.Models.ImageUrl} that contains the
   * url of the image to be evaluated.
   *
   * @param {string} imageUrl.url Url of the image.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ImagePrediction>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  detectImageUrlWithNoStoreWithHttpOperationResponse(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ImagePrediction>>;

  /**
   * @summary Detect objects in an image url without saving the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageUrl An {Iris.Web.Api.Models.ImageUrl} that contains the
   * url of the image to be evaluated.
   *
   * @param {string} imageUrl.url Url of the image.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ImagePrediction} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ImagePrediction} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ImagePrediction} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  detectImageUrlWithNoStore(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ImagePrediction>;
  detectImageUrlWithNoStore(projectId: string, publishedName: string, imageUrl: models.ImageUrl, callback: ServiceCallback<models.ImagePrediction>): void;
  detectImageUrlWithNoStore(projectId: string, publishedName: string, imageUrl: models.ImageUrl, options: { application? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ImagePrediction>): void;


  /**
   * @summary Detect objects in an image without saving the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageData Binary image data. Supported formats are JPEG,
   * GIF, PNG, and BMP. Supports images up to 0MB.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ImagePrediction>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  detectImageWithNoStoreWithHttpOperationResponse(projectId: string, publishedName: string, imageData: stream.Readable, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ImagePrediction>>;

  /**
   * @summary Detect objects in an image without saving the result.
   *
   * @param {uuid} projectId The project id.
   *
   * @param {string} publishedName Specifies the name of the model to evaluate
   * against.
   *
   * @param {object} imageData Binary image data. Supported formats are JPEG,
   * GIF, PNG, and BMP. Supports images up to 0MB.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.application] Optional. Specifies the name of
   * application using the endpoint.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ImagePrediction} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ImagePrediction} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ImagePrediction} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  detectImageWithNoStore(projectId: string, publishedName: string, imageData: stream.Readable, options?: { application? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ImagePrediction>;
  detectImageWithNoStore(projectId: string, publishedName: string, imageData: stream.Readable, callback: ServiceCallback<models.ImagePrediction>): void;
  detectImageWithNoStore(projectId: string, publishedName: string, imageData: stream.Readable, options: { application? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ImagePrediction>): void;
}

export { PredictionAPIClient, models as PredictionAPIModels };
