/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Information of the sku.
 *
 */
class SkuInformation {
  /**
   * Create a SkuInformation.
   * @member {object} [sku] The Sku.
   * @member {string} [sku.name] The sku name. Possible values include:
   * 'DataBox', 'DataBoxDisk', 'DataBoxHeavy'
   * @member {string} [sku.displayName] The display name of the sku.
   * @member {string} [sku.family] The sku family.
   * @member {boolean} [enabled] The sku is enabled or not.
   * @member {array} [destinationToServiceLocationMap] The map of destination
   * location to service location.
   * @member {object} [capacity] Capacity of the Sku.
   * @member {string} [capacity.usable] Usable capacity in TB.
   * @member {string} [capacity.maximum] Maximum capacity in TB.
   * @member {array} [costs] Cost of the Sku.
   * @member {array} [apiVersions] Api versions that support this Sku.
   * @member {string} [disabledReason] Reason why the Sku is disabled. Possible
   * values include: 'None', 'Country', 'Region', 'Feature', 'OfferType'
   * @member {string} [disabledReasonMessage] Message for why the Sku is
   * disabled.
   * @member {string} [requiredFeature] Required feature to access the sku.
   */
  constructor() {
  }

  /**
   * Defines the metadata of SkuInformation
   *
   * @returns {object} metadata of SkuInformation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'SkuInformation',
      type: {
        name: 'Composite',
        className: 'SkuInformation',
        modelProperties: {
          sku: {
            required: false,
            readOnly: true,
            serializedName: 'sku',
            type: {
              name: 'Composite',
              className: 'Sku'
            }
          },
          enabled: {
            required: false,
            readOnly: true,
            serializedName: 'enabled',
            type: {
              name: 'Boolean'
            }
          },
          destinationToServiceLocationMap: {
            required: false,
            readOnly: true,
            serializedName: 'properties.destinationToServiceLocationMap',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'DestinationToServiceLocationMapElementType',
                  type: {
                    name: 'Composite',
                    className: 'DestinationToServiceLocationMap'
                  }
              }
            }
          },
          capacity: {
            required: false,
            readOnly: true,
            serializedName: 'properties.capacity',
            type: {
              name: 'Composite',
              className: 'SkuCapacity'
            }
          },
          costs: {
            required: false,
            readOnly: true,
            serializedName: 'properties.costs',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'SkuCostElementType',
                  type: {
                    name: 'Composite',
                    className: 'SkuCost'
                  }
              }
            }
          },
          apiVersions: {
            required: false,
            readOnly: true,
            serializedName: 'properties.apiVersions',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          disabledReason: {
            required: false,
            readOnly: true,
            serializedName: 'properties.disabledReason',
            type: {
              name: 'Enum',
              allowedValues: [ 'None', 'Country', 'Region', 'Feature', 'OfferType' ]
            }
          },
          disabledReasonMessage: {
            required: false,
            readOnly: true,
            serializedName: 'properties.disabledReasonMessage',
            type: {
              name: 'String'
            }
          },
          requiredFeature: {
            required: false,
            readOnly: true,
            serializedName: 'properties.requiredFeature',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = SkuInformation;
