/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Microsoft Azure SQL Database linked service.
 *
 * @extends models['LinkedService']
 */
class AzureSqlDatabaseLinkedService extends models['LinkedService'] {
  /**
   * Create a AzureSqlDatabaseLinkedService.
   * @member {object} connectionString The connection string. Type: string,
   * SecureString or AzureKeyVaultSecretReference.
   * @member {object} [password] The Azure key vault secret reference of
   * password in connection string.
   * @member {object} [password.store] The Azure Key Vault linked service
   * reference.
   * @member {string} [password.store.referenceName] Reference LinkedService
   * name.
   * @member {object} [password.store.parameters] Arguments for LinkedService.
   * @member {object} [password.secretName] The name of the secret in Azure Key
   * Vault. Type: string (or Expression with resultType string).
   * @member {object} [password.secretVersion] The version of the secret in
   * Azure Key Vault. The default value is the latest version of the secret.
   * Type: string (or Expression with resultType string).
   * @member {object} [servicePrincipalId] The ID of the service principal used
   * to authenticate against Azure SQL Database. Type: string (or Expression
   * with resultType string).
   * @member {object} [servicePrincipalKey] The key of the service principal
   * used to authenticate against Azure SQL Database.
   * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
   * @member {object} [tenant] The name or ID of the tenant to which the
   * service principal belongs. Type: string (or Expression with resultType
   * string).
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AzureSqlDatabaseLinkedService
   *
   * @returns {object} metadata of AzureSqlDatabaseLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AzureSqlDatabase',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'AzureSqlDatabaseLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          connectionString: {
            required: true,
            serializedName: 'typeProperties.connectionString',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'AzureKeyVaultSecretReference'
            }
          },
          servicePrincipalId: {
            required: false,
            serializedName: 'typeProperties.servicePrincipalId',
            type: {
              name: 'Object'
            }
          },
          servicePrincipalKey: {
            required: false,
            serializedName: 'typeProperties.servicePrincipalKey',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          tenant: {
            required: false,
            serializedName: 'typeProperties.tenant',
            type: {
              name: 'Object'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = AzureSqlDatabaseLinkedService;
