/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Dynamics AX linked service.
 *
 * @extends models['LinkedService']
 */
class DynamicsAXLinkedService extends models['LinkedService'] {
  /**
   * Create a DynamicsAXLinkedService.
   * @member {object} url The URL of Dynamics AX OData API. Type: string (or
   * Expression with resultType string).
   * @member {object} [tenant] Specify the tenant information (domain name or
   * tenant ID) under which your application resides. Type: string (or
   * Expression with resultType string).
   * @member {object} [servicePrincipalId] Specify the application id of your
   * application registered in Azure Active Directory. Type: string (or
   * Expression with resultType string).
   * @member {object} [aadResourceId] Specify the resource you are requesting
   * authorization to use Directory. Type: string (or Expression with
   * resultType string).
   * @member {object} [servicePrincipalKey] Specify the secret of your
   * application registered in Azure Active Directory. Type: string (or
   * Expression with resultType string).
   * @member {string} [servicePrincipalKey.type] Polymorphic Discriminator
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Either encryptedCredential or username/password must
   * be provided. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of DynamicsAXLinkedService
   *
   * @returns {object} metadata of DynamicsAXLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'DynamicsAX',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'type',
          clientName: 'type'
        },
        uberParent: 'LinkedService',
        className: 'DynamicsAXLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          annotations: {
            required: false,
            serializedName: 'annotations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          url: {
            required: true,
            serializedName: 'typeProperties.url',
            type: {
              name: 'Object'
            }
          },
          tenant: {
            required: false,
            serializedName: 'typeProperties.tenant',
            type: {
              name: 'Object'
            }
          },
          servicePrincipalId: {
            required: false,
            serializedName: 'typeProperties.servicePrincipalId',
            type: {
              name: 'Object'
            }
          },
          aadResourceId: {
            required: false,
            serializedName: 'typeProperties.aadResourceId',
            type: {
              name: 'Object'
            }
          },
          servicePrincipalKey: {
            required: false,
            serializedName: 'typeProperties.servicePrincipalKey',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'SecretBase',
              className: 'SecretBase'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = DynamicsAXLinkedService;
