/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The resource that defines the source location where the artifacts are
 * located.
 *
 * @extends models['TrackedResource']
 */
class ArtifactSource extends models['TrackedResource'] {
  /**
   * Create a ArtifactSource.
   * @member {string} sourceType The type of artifact source used.
   * @member {string} [artifactRoot] The path from the location that the
   * 'authentication' property [say, a SAS URI to the blob container] refers
   * to, to the location of the artifacts. This can be used to differentiate
   * different versions of the artifacts. Or, different types of artifacts like
   * binaries or templates. The location referenced by the authentication
   * property concatenated with this optional artifactRoot path forms the
   * artifact source location where the artifacts are expected to be found.
   * @member {object} authentication The authentication method to use to access
   * the artifact source.
   * @member {string} [authentication.type] Polymorphic Discriminator
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ArtifactSource
   *
   * @returns {object} metadata of ArtifactSource
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ArtifactSource',
      type: {
        name: 'Composite',
        className: 'ArtifactSource',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          sourceType: {
            required: true,
            serializedName: 'properties.sourceType',
            type: {
              name: 'String'
            }
          },
          artifactRoot: {
            required: false,
            serializedName: 'properties.artifactRoot',
            type: {
              name: 'String'
            }
          },
          authentication: {
            required: true,
            serializedName: 'properties.authentication',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'Authentication',
              className: 'Authentication'
            }
          }
        }
      }
    };
  }
}

module.exports = ArtifactSource;
