/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Defines information about a local entity, such as a restaurant or hotel.
 *
 * @extends models['Thing']
 */
class Place extends models['Thing'] {
  /**
   * Create a Place.
   * @property {object} [address] The postal address of where the entity is
   * located
   * @property {string} [address.streetAddress]
   * @property {string} [address.addressLocality] The city where the street
   * address is located. For example, Seattle.
   * @property {string} [address.addressSubregion]
   * @property {string} [address.addressRegion] The state or province code
   * where the street address is located. This could be the two-letter code.
   * For example, WA, or the full name , Washington.
   * @property {string} [address.postalCode] The zip code or postal code where
   * the street address is located. For example, 98052.
   * @property {string} [address.postOfficeBoxNumber]
   * @property {string} [address.addressCountry] The country/region where the
   * street address is located. This could be the two-letter ISO code. For
   * example, US, or the full name, United States.
   * @property {string} [address.countryIso] The two letter ISO code of this
   * country. For example, US.
   * @property {string} [address.neighborhood] The neighborhood where the
   * street address is located. For example, Westlake.
   * @property {string} [address.addressRegionAbbreviation] Region
   * Abbreviation. For example, WA.
   * @property {string} [address.text] The complete address. For example, 2100
   * Westlake Ave N, Bellevue, WA 98052.
   * @property {string} [telephone] The entity's telephone number
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Place
   *
   * @returns {object} metadata of Place
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Place',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: '_type',
          clientName: '_type'
        },
        uberParent: 'ResponseBase',
        className: 'Place',
        modelProperties: {
          _type: {
            required: true,
            serializedName: '_type',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          contractualRules: {
            required: false,
            readOnly: true,
            serializedName: 'contractualRules',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ContractualRulesContractualRuleElementType',
                  type: {
                    name: 'Composite',
                    polymorphicDiscriminator: {
                      serializedName: '_type',
                      clientName: '_type'
                    },
                    uberParent: 'ContractualRulesContractualRule',
                    className: 'ContractualRulesContractualRule'
                  }
              }
            }
          },
          webSearchUrl: {
            required: false,
            readOnly: true,
            serializedName: 'webSearchUrl',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          url: {
            required: false,
            readOnly: true,
            serializedName: 'url',
            type: {
              name: 'String'
            }
          },
          image: {
            required: false,
            readOnly: true,
            serializedName: 'image',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'ResponseBase',
              className: 'ImageObject'
            }
          },
          description: {
            required: false,
            readOnly: true,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          entityPresentationInfo: {
            required: false,
            readOnly: true,
            serializedName: 'entityPresentationInfo',
            type: {
              name: 'Composite',
              className: 'EntitiesEntityPresentationInfo'
            }
          },
          bingId: {
            required: false,
            readOnly: true,
            serializedName: 'bingId',
            type: {
              name: 'String'
            }
          },
          address: {
            required: false,
            readOnly: true,
            serializedName: 'address',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '_type',
                clientName: '_type'
              },
              uberParent: 'ResponseBase',
              className: 'PostalAddress'
            }
          },
          telephone: {
            required: false,
            readOnly: true,
            serializedName: 'telephone',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = Place;
