/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Schema of the Data property of an EventGridEvent for a
 * Microsoft.Media.JobOutputStateChange event.
 *
 */
class MediaJobOutputStateChangeEventData {
  /**
   * Create a MediaJobOutputStateChangeEventData.
   * @member {string} [previousState] The previous state of the Job. Possible
   * values include: 'Canceled', 'Canceling', 'Error', 'Finished',
   * 'Processing', 'Queued', 'Scheduled'
   * @member {object} [output] Gets the output.
   * @member {object} [output.error] Gets the Job output error.
   * @member {string} [output.error.code] Error code describing the error.
   * Possible values include: 'ServiceError', 'ServiceTransientError',
   * 'DownloadNotAccessible', 'DownloadTransientError', 'UploadNotAccessible',
   * 'UploadTransientError', 'ConfigurationUnsupported', 'ContentMalformed',
   * 'ContentUnsupported'
   * @member {string} [output.error.message] A human-readable
   * language-dependent representation of the error.
   * @member {string} [output.error.category] Helps with categorization of
   * errors. Possible values include: 'Service', 'Download', 'Upload',
   * 'Configuration', 'Content'
   * @member {string} [output.error.retry] Indicates that it may be possible to
   * retry the Job. If retry is unsuccessful, please contact Azure support via
   * Azure Portal. Possible values include: 'DoNotRetry', 'MayRetry'
   * @member {array} [output.error.details] An array of details about specific
   * errors that led to this reported error.
   * @member {string} [output.label] Gets the Job output label.
   * @member {number} [output.progress] Gets the Job output progress.
   * @member {string} [output.state] Gets the Job output state. Possible values
   * include: 'Canceled', 'Canceling', 'Error', 'Finished', 'Processing',
   * 'Queued', 'Scheduled'
   * @member {string} [output.odatatype] Polymorphic Discriminator
   * @member {object} [jobCorrelationData] Gets the Job correlation data.
   */
  constructor() {
  }

  /**
   * Defines the metadata of MediaJobOutputStateChangeEventData
   *
   * @returns {object} metadata of MediaJobOutputStateChangeEventData
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'MediaJobOutputStateChangeEventData',
      type: {
        name: 'Composite',
        className: 'MediaJobOutputStateChangeEventData',
        modelProperties: {
          previousState: {
            required: false,
            nullable: false,
            readOnly: true,
            serializedName: 'previousState',
            type: {
              name: 'Enum',
              allowedValues: [ 'Canceled', 'Canceling', 'Error', 'Finished', 'Processing', 'Queued', 'Scheduled' ]
            }
          },
          output: {
            required: false,
            serializedName: 'output',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: '@odata.type',
                clientName: 'odatatype'
              },
              uberParent: 'MediaJobOutput',
              className: 'MediaJobOutput'
            }
          },
          jobCorrelationData: {
            required: false,
            serializedName: 'jobCorrelationData',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = MediaJobOutputStateChangeEventData;
