/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * The properties of cluster.
 *
 */
class ClusterGetProperties {
  /**
   * Create a ClusterGetProperties.
   * @property {string} [clusterVersion] The version of the cluster.
   * @property {string} [osType] The type of operating system. Possible values
   * include: 'Windows', 'Linux'
   * @property {string} [tier] The cluster tier. Possible values include:
   * 'Standard', 'Premium'
   * @property {object} clusterDefinition The cluster definition.
   * @property {string} [clusterDefinition.blueprint] The link to the
   * blueprint.
   * @property {string} [clusterDefinition.kind] The type of cluster.
   * @property {object} [clusterDefinition.componentVersion] The versions of
   * different services in the cluster.
   * @property {object} [clusterDefinition.configurations] The cluster
   * configurations.
   * @property {object} [securityProfile] The security profile.
   * @property {string} [securityProfile.directoryType] The directory type.
   * Possible values include: 'ActiveDirectory'
   * @property {string} [securityProfile.domain] The organization's active
   * directory domain.
   * @property {string} [securityProfile.organizationalUnitDN] The
   * organizational unit within the Active Directory to place the cluster and
   * service accounts.
   * @property {array} [securityProfile.ldapsUrls] The LDAPS protocol URLs to
   * communicate with the Active Directory.
   * @property {string} [securityProfile.domainUsername] The domain user
   * account that will have admin privileges on the cluster.
   * @property {string} [securityProfile.domainUserPassword] The domain admin
   * password.
   * @property {array} [securityProfile.clusterUsersGroupDNs] Optional. The
   * Distinguished Names for cluster user groups
   * @property {string} [securityProfile.aaddsResourceId] The resource ID of
   * the user's Azure Active Directory Domain Service.
   * @property {string} [securityProfile.msiResourceId] User assigned identity
   * that has permissions to read and create cluster-related artifacts in the
   * user's AADDS.
   * @property {object} [computeProfile] The compute profile.
   * @property {array} [computeProfile.roles] The list of roles in the cluster.
   * @property {string} [provisioningState] The provisioning state, which only
   * appears in the response. Possible values include: 'InProgress', 'Failed',
   * 'Succeeded', 'Canceled', 'Deleting'
   * @property {string} [createdDate] The date on which the cluster was
   * created.
   * @property {string} [clusterState] The state of the cluster.
   * @property {object} [quotaInfo] The quota information.
   * @property {number} [quotaInfo.coresUsed] The cores used by the cluster.
   * @property {array} [errors] The list of errors.
   * @property {array} [connectivityEndpoints] The list of connectivity
   * endpoints.
   * @property {object} [diskEncryptionProperties] The disk encryption
   * properties.
   * @property {string} [diskEncryptionProperties.vaultUri] Base key vault URI
   * where the customers key is located eg. https://myvault.vault.azure.net
   * @property {string} [diskEncryptionProperties.keyName] Key name that is
   * used for enabling disk encryption.
   * @property {string} [diskEncryptionProperties.keyVersion] Specific key
   * version that is used for enabling disk encryption.
   * @property {string} [diskEncryptionProperties.encryptionAlgorithm]
   * Algorithm identifier for encryption, default RSA-OAEP. Possible values
   * include: 'RSA-OAEP', 'RSA-OAEP-256', 'RSA1_5'
   * @property {string} [diskEncryptionProperties.msiResourceId] Resource ID of
   * Managed Identity that is used to access the key vault.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ClusterGetProperties
   *
   * @returns {object} metadata of ClusterGetProperties
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ClusterGetProperties',
      type: {
        name: 'Composite',
        className: 'ClusterGetProperties',
        modelProperties: {
          clusterVersion: {
            required: false,
            serializedName: 'clusterVersion',
            type: {
              name: 'String'
            }
          },
          osType: {
            required: false,
            serializedName: 'osType',
            type: {
              name: 'Enum',
              allowedValues: [ 'Windows', 'Linux' ]
            }
          },
          tier: {
            required: false,
            serializedName: 'tier',
            type: {
              name: 'Enum',
              allowedValues: [ 'Standard', 'Premium' ]
            }
          },
          clusterDefinition: {
            required: true,
            serializedName: 'clusterDefinition',
            type: {
              name: 'Composite',
              className: 'ClusterDefinition'
            }
          },
          securityProfile: {
            required: false,
            serializedName: 'securityProfile',
            type: {
              name: 'Composite',
              className: 'SecurityProfile'
            }
          },
          computeProfile: {
            required: false,
            serializedName: 'computeProfile',
            type: {
              name: 'Composite',
              className: 'ComputeProfile'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'provisioningState',
            type: {
              name: 'Enum',
              allowedValues: [ 'InProgress', 'Failed', 'Succeeded', 'Canceled', 'Deleting' ]
            }
          },
          createdDate: {
            required: false,
            serializedName: 'createdDate',
            type: {
              name: 'String'
            }
          },
          clusterState: {
            required: false,
            serializedName: 'clusterState',
            type: {
              name: 'String'
            }
          },
          quotaInfo: {
            required: false,
            serializedName: 'quotaInfo',
            type: {
              name: 'Composite',
              className: 'QuotaInfo'
            }
          },
          errors: {
            required: false,
            serializedName: 'errors',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ErrorsElementType',
                  type: {
                    name: 'Composite',
                    className: 'Errors'
                  }
              }
            }
          },
          connectivityEndpoints: {
            required: false,
            serializedName: 'connectivityEndpoints',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ConnectivityEndpointElementType',
                  type: {
                    name: 'Composite',
                    className: 'ConnectivityEndpoint'
                  }
              }
            }
          },
          diskEncryptionProperties: {
            required: false,
            serializedName: 'diskEncryptionProperties',
            type: {
              name: 'Composite',
              className: 'DiskEncryptionProperties'
            }
          }
        }
      }
    };
  }
}

module.exports = ClusterGetProperties;
