/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A Machine Learning compute based on Azure Virtual Machines.
 *
 * @extends models['Compute']
 */
class VirtualMachine extends models['Compute'] {
  /**
   * Create a VirtualMachine.
   * @member {object} [properties]
   * @member {string} [properties.virtualMachineSize] Virtual Machine size
   * @member {number} [properties.sshPort] Port open for ssh connections.
   * @member {string} [properties.address] Public IP address of the virtual
   * machine.
   * @member {object} [properties.administratorAccount] Admin credentials for
   * virtual machine
   * @member {string} [properties.administratorAccount.username] Username of
   * admin account
   * @member {string} [properties.administratorAccount.password] Password of
   * admin account
   * @member {string} [properties.administratorAccount.publicKeyData] Public
   * key data
   * @member {string} [properties.administratorAccount.privateKeyData] Private
   * key data
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of VirtualMachine
   *
   * @returns {object} metadata of VirtualMachine
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'VirtualMachine',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'computeType',
          clientName: 'computeType'
        },
        uberParent: 'Compute',
        className: 'VirtualMachine',
        modelProperties: {
          computeLocation: {
            required: false,
            serializedName: 'computeLocation',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'provisioningState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          createdOn: {
            required: false,
            readOnly: true,
            serializedName: 'createdOn',
            type: {
              name: 'DateTime'
            }
          },
          modifiedOn: {
            required: false,
            readOnly: true,
            serializedName: 'modifiedOn',
            type: {
              name: 'DateTime'
            }
          },
          resourceId: {
            required: false,
            serializedName: 'resourceId',
            type: {
              name: 'String'
            }
          },
          provisioningErrors: {
            required: false,
            readOnly: true,
            serializedName: 'provisioningErrors',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'MachineLearningServiceErrorElementType',
                  type: {
                    name: 'Composite',
                    className: 'MachineLearningServiceError'
                  }
              }
            }
          },
          computeType: {
            required: true,
            serializedName: 'computeType',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          properties: {
            required: false,
            serializedName: 'properties',
            type: {
              name: 'Composite',
              className: 'VirtualMachineProperties'
            }
          }
        }
      }
    };
  }
}

module.exports = VirtualMachine;
