/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMachineLearningWorkspaces.
 */
export interface Operations {


    /**
     * Lists all of the available Azure Machine Learning Workspaces REST API
     * operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available Azure Machine Learning Workspaces REST API
     * operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    list(callback: ServiceCallback<models.OperationListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;
}

/**
 * @class
 * Workspaces
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMachineLearningWorkspaces.
 */
export interface Workspaces {


    /**
     * Gets the properties of the specified machine learning workspace.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Workspace>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Workspace>>;

    /**
     * Gets the properties of the specified machine learning workspace.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Workspace} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Workspace} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Workspace} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, workspaceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Workspace>;
    get(resourceGroupName: string, workspaceName: string, callback: ServiceCallback<models.Workspace>): void;
    get(resourceGroupName: string, workspaceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Workspace>): void;


    /**
     * Creates or updates a workspace with the specified parameters.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} parameters The parameters for creating or updating a machine
     * learning workspace.
     *
     * @param {string} [parameters.description] The description of this workspace.
     *
     * @param {string} [parameters.friendlyName] The friendly name for this
     * workspace. This name in mutable
     *
     * @param {string} [parameters.batchaiWorkspace] ARM id of the Batch AI
     * workspace associated with this workspace. This cannot be changed once the
     * workspace has been created
     *
     * @param {string} [parameters.keyVault] ARM id of the key vault associated
     * with this workspace. This cannot be changed once the workspace has been
     * created
     *
     * @param {string} [parameters.applicationInsights] ARM id of the application
     * insights associated with this workspace. This cannot be changed once the
     * workspace has been created
     *
     * @param {string} [parameters.containerRegistry] ARM id of the container
     * registry associated with this workspace. This cannot be changed once the
     * workspace has been created
     *
     * @param {string} [parameters.storageAccount] ARM id of the storage account
     * associated with this workspace. This cannot be changed once the workspace
     * has been created
     *
     * @param {string} [parameters.discoveryUrl] Url for the discovery service to
     * identify regional endpoints for machine learning experimentation services
     *
     * @param {string} [parameters.location] Specifies the location of the
     * resource.
     *
     * @param {object} [parameters.tags] Contains resource tags defined as
     * key/value pairs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Workspace>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, parameters: models.Workspace, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Workspace>>;

    /**
     * Creates or updates a workspace with the specified parameters.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} parameters The parameters for creating or updating a machine
     * learning workspace.
     *
     * @param {string} [parameters.description] The description of this workspace.
     *
     * @param {string} [parameters.friendlyName] The friendly name for this
     * workspace. This name in mutable
     *
     * @param {string} [parameters.batchaiWorkspace] ARM id of the Batch AI
     * workspace associated with this workspace. This cannot be changed once the
     * workspace has been created
     *
     * @param {string} [parameters.keyVault] ARM id of the key vault associated
     * with this workspace. This cannot be changed once the workspace has been
     * created
     *
     * @param {string} [parameters.applicationInsights] ARM id of the application
     * insights associated with this workspace. This cannot be changed once the
     * workspace has been created
     *
     * @param {string} [parameters.containerRegistry] ARM id of the container
     * registry associated with this workspace. This cannot be changed once the
     * workspace has been created
     *
     * @param {string} [parameters.storageAccount] ARM id of the storage account
     * associated with this workspace. This cannot be changed once the workspace
     * has been created
     *
     * @param {string} [parameters.discoveryUrl] Url for the discovery service to
     * identify regional endpoints for machine learning experimentation services
     *
     * @param {string} [parameters.location] Specifies the location of the
     * resource.
     *
     * @param {object} [parameters.tags] Contains resource tags defined as
     * key/value pairs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Workspace} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Workspace} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Workspace} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, workspaceName: string, parameters: models.Workspace, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Workspace>;
    createOrUpdate(resourceGroupName: string, workspaceName: string, parameters: models.Workspace, callback: ServiceCallback<models.Workspace>): void;
    createOrUpdate(resourceGroupName: string, workspaceName: string, parameters: models.Workspace, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Workspace>): void;


    /**
     * Deletes a machine learning workspace.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a machine learning workspace.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, workspaceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, workspaceName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, workspaceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a machine learning workspace with the specified parameters.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} parameters The parameters for updating a machine learning
     * workspace.
     *
     * @param {object} [parameters.tags] The resource tags for the machine learning
     * workspace.
     *
     * @param {string} [parameters.description] The description of this workspace.
     *
     * @param {string} [parameters.friendlyName] The friendly name for this
     * workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Workspace>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, parameters: models.WorkspaceUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Workspace>>;

    /**
     * Updates a machine learning workspace with the specified parameters.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} parameters The parameters for updating a machine learning
     * workspace.
     *
     * @param {object} [parameters.tags] The resource tags for the machine learning
     * workspace.
     *
     * @param {string} [parameters.description] The description of this workspace.
     *
     * @param {string} [parameters.friendlyName] The friendly name for this
     * workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Workspace} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Workspace} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Workspace} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, workspaceName: string, parameters: models.WorkspaceUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Workspace>;
    update(resourceGroupName: string, workspaceName: string, parameters: models.WorkspaceUpdateParameters, callback: ServiceCallback<models.Workspace>): void;
    update(resourceGroupName: string, workspaceName: string, parameters: models.WorkspaceUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Workspace>): void;


    /**
     * Lists all the available machine learning workspaces under the specified
     * resource group.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skiptoken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceListResult>>;

    /**
     * Lists all the available machine learning workspaces under the specified
     * resource group.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skiptoken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceListResult>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.WorkspaceListResult>): void;
    listByResourceGroup(resourceGroupName: string, options: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceListResult>): void;


    /**
     * Lists all the keys associated with this workspace. This includes keys for
     * the storage account, app insights and password for container registry
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ListWorkspaceKeysResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listKeysWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ListWorkspaceKeysResult>>;

    /**
     * Lists all the keys associated with this workspace. This includes keys for
     * the storage account, app insights and password for container registry
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ListWorkspaceKeysResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ListWorkspaceKeysResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ListWorkspaceKeysResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listKeys(resourceGroupName: string, workspaceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ListWorkspaceKeysResult>;
    listKeys(resourceGroupName: string, workspaceName: string, callback: ServiceCallback<models.ListWorkspaceKeysResult>): void;
    listKeys(resourceGroupName: string, workspaceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ListWorkspaceKeysResult>): void;


    /**
     * Resync all the keys associated with this workspace. This includes keys for
     * the storage account, app insights and password for container registry
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    resyncKeysWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Resync all the keys associated with this workspace. This includes keys for
     * the storage account, app insights and password for container registry
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    resyncKeys(resourceGroupName: string, workspaceName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    resyncKeys(resourceGroupName: string, workspaceName: string, callback: ServiceCallback<void>): void;
    resyncKeys(resourceGroupName: string, workspaceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Lists all the available machine learning workspaces under the specified
     * subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skiptoken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionWithHttpOperationResponse(options?: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceListResult>>;

    /**
     * Lists all the available machine learning workspaces under the specified
     * subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skiptoken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscription(options?: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceListResult>;
    listBySubscription(callback: ServiceCallback<models.WorkspaceListResult>): void;
    listBySubscription(options: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceListResult>): void;


    /**
     * Lists all the available machine learning workspaces under the specified
     * resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceListResult>>;

    /**
     * Lists all the available machine learning workspaces under the specified
     * resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceListResult>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.WorkspaceListResult>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceListResult>): void;


    /**
     * Lists all the available machine learning workspaces under the specified
     * subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceListResult>>;

    /**
     * Lists all the available machine learning workspaces under the specified
     * subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscriptionNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceListResult>;
    listBySubscriptionNext(nextPageLink: string, callback: ServiceCallback<models.WorkspaceListResult>): void;
    listBySubscriptionNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceListResult>): void;
}

/**
 * @class
 * MachineLearningCompute
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMachineLearningWorkspaces.
 */
export interface MachineLearningCompute {


    /**
     * Gets computes in specified workspace.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skiptoken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<PaginatedComputeResourcesList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByWorkspaceWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, options?: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.PaginatedComputeResourcesList>>;

    /**
     * Gets computes in specified workspace.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skiptoken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {PaginatedComputeResourcesList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {PaginatedComputeResourcesList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link PaginatedComputeResourcesList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByWorkspace(resourceGroupName: string, workspaceName: string, options?: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.PaginatedComputeResourcesList>;
    listByWorkspace(resourceGroupName: string, workspaceName: string, callback: ServiceCallback<models.PaginatedComputeResourcesList>): void;
    listByWorkspace(resourceGroupName: string, workspaceName: string, options: { skiptoken? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.PaginatedComputeResourcesList>): void;


    /**
     * Gets compute definition by its name. Any secrets (storage keys, service
     * credentials, etc) are not returned - use 'keys' nested resource to get them.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ComputeResource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ComputeResource>>;

    /**
     * Gets compute definition by its name. Any secrets (storage keys, service
     * credentials, etc) are not returned - use 'keys' nested resource to get them.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ComputeResource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ComputeResource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ComputeResource} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ComputeResource>;
    get(resourceGroupName: string, workspaceName: string, computeName: string, callback: ServiceCallback<models.ComputeResource>): void;
    get(resourceGroupName: string, workspaceName: string, computeName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ComputeResource>): void;


    /**
     * Creates or updates compute. This call will overwrite a compute if it exists.
     * This is a nonrecoverable operation. If your intent is to create a new
     * compute, do a GET first to verify that it does not exist yet.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} parameters Payload with Machine Learning compute definition.
     *
     * @param {object} [parameters.properties] Compute properties
     *
     * @param {string} [parameters.properties.computeLocation] Location for the
     * underlying compute
     *
     * @param {string} [parameters.properties.description] The description of the
     * Machine Learning compute.
     *
     * @param {string} [parameters.properties.resourceId] ARM resource id of the
     * compute
     *
     * @param {string} parameters.properties.computeType Polymorphic Discriminator
     *
     * @param {string} [parameters.location] Specifies the location of the
     * resource.
     *
     * @param {object} [parameters.tags] Contains resource tags defined as
     * key/value pairs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ComputeResource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, computeName: string, parameters: models.ComputeResource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ComputeResource>>;

    /**
     * Creates or updates compute. This call will overwrite a compute if it exists.
     * This is a nonrecoverable operation. If your intent is to create a new
     * compute, do a GET first to verify that it does not exist yet.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} parameters Payload with Machine Learning compute definition.
     *
     * @param {object} [parameters.properties] Compute properties
     *
     * @param {string} [parameters.properties.computeLocation] Location for the
     * underlying compute
     *
     * @param {string} [parameters.properties.description] The description of the
     * Machine Learning compute.
     *
     * @param {string} [parameters.properties.resourceId] ARM resource id of the
     * compute
     *
     * @param {string} parameters.properties.computeType Polymorphic Discriminator
     *
     * @param {string} [parameters.location] Specifies the location of the
     * resource.
     *
     * @param {object} [parameters.tags] Contains resource tags defined as
     * key/value pairs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ComputeResource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ComputeResource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ComputeResource} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, workspaceName: string, computeName: string, parameters: models.ComputeResource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ComputeResource>;
    createOrUpdate(resourceGroupName: string, workspaceName: string, computeName: string, parameters: models.ComputeResource, callback: ServiceCallback<models.ComputeResource>): void;
    createOrUpdate(resourceGroupName: string, workspaceName: string, computeName: string, parameters: models.ComputeResource, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ComputeResource>): void;


    /**
     * Deletes specified Machine Learning compute.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes specified Machine Learning compute.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, workspaceName: string, computeName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, workspaceName: string, computeName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * System Update On Machine Learning compute.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    systemUpdateWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * System Update On Machine Learning compute.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    systemUpdate(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    systemUpdate(resourceGroupName: string, workspaceName: string, computeName: string, callback: ServiceCallback<void>): void;
    systemUpdate(resourceGroupName: string, workspaceName: string, computeName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Gets secrets related to Machine Learning compute (storage keys, service
     * credentials, etc).
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ComputeSecrets>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listKeysWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ComputeSecrets>>;

    /**
     * Gets secrets related to Machine Learning compute (storage keys, service
     * credentials, etc).
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ComputeSecrets} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ComputeSecrets} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ComputeSecrets} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listKeys(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ComputeSecrets>;
    listKeys(resourceGroupName: string, workspaceName: string, computeName: string, callback: ServiceCallback<models.ComputeSecrets>): void;
    listKeys(resourceGroupName: string, workspaceName: string, computeName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ComputeSecrets>): void;


    /**
     * Creates or updates compute. This call will overwrite a compute if it exists.
     * This is a nonrecoverable operation. If your intent is to create a new
     * compute, do a GET first to verify that it does not exist yet.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} parameters Payload with Machine Learning compute definition.
     *
     * @param {object} [parameters.properties] Compute properties
     *
     * @param {string} [parameters.properties.computeLocation] Location for the
     * underlying compute
     *
     * @param {string} [parameters.properties.description] The description of the
     * Machine Learning compute.
     *
     * @param {string} [parameters.properties.resourceId] ARM resource id of the
     * compute
     *
     * @param {string} parameters.properties.computeType Polymorphic Discriminator
     *
     * @param {string} [parameters.location] Specifies the location of the
     * resource.
     *
     * @param {object} [parameters.tags] Contains resource tags defined as
     * key/value pairs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ComputeResource>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, computeName: string, parameters: models.ComputeResource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ComputeResource>>;

    /**
     * Creates or updates compute. This call will overwrite a compute if it exists.
     * This is a nonrecoverable operation. If your intent is to create a new
     * compute, do a GET first to verify that it does not exist yet.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} parameters Payload with Machine Learning compute definition.
     *
     * @param {object} [parameters.properties] Compute properties
     *
     * @param {string} [parameters.properties.computeLocation] Location for the
     * underlying compute
     *
     * @param {string} [parameters.properties.description] The description of the
     * Machine Learning compute.
     *
     * @param {string} [parameters.properties.resourceId] ARM resource id of the
     * compute
     *
     * @param {string} parameters.properties.computeType Polymorphic Discriminator
     *
     * @param {string} [parameters.location] Specifies the location of the
     * resource.
     *
     * @param {object} [parameters.tags] Contains resource tags defined as
     * key/value pairs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ComputeResource} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ComputeResource} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ComputeResource} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreateOrUpdate(resourceGroupName: string, workspaceName: string, computeName: string, parameters: models.ComputeResource, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ComputeResource>;
    beginCreateOrUpdate(resourceGroupName: string, workspaceName: string, computeName: string, parameters: models.ComputeResource, callback: ServiceCallback<models.ComputeResource>): void;
    beginCreateOrUpdate(resourceGroupName: string, workspaceName: string, computeName: string, parameters: models.ComputeResource, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ComputeResource>): void;


    /**
     * Deletes specified Machine Learning compute.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes specified Machine Learning compute.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, workspaceName: string, computeName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, workspaceName: string, computeName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * System Update On Machine Learning compute.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginSystemUpdateWithHttpOperationResponse(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * System Update On Machine Learning compute.
     *
     * @param {string} resourceGroupName Name of the resource group in which
     * workspace is located.
     *
     * @param {string} workspaceName Name of Azure Machine Learning workspace.
     *
     * @param {string} computeName Name of the Azure Machine Learning compute.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginSystemUpdate(resourceGroupName: string, workspaceName: string, computeName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginSystemUpdate(resourceGroupName: string, workspaceName: string, computeName: string, callback: ServiceCallback<void>): void;
    beginSystemUpdate(resourceGroupName: string, workspaceName: string, computeName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Gets computes in specified workspace.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<PaginatedComputeResourcesList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByWorkspaceNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.PaginatedComputeResourcesList>>;

    /**
     * Gets computes in specified workspace.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {PaginatedComputeResourcesList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {PaginatedComputeResourcesList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link PaginatedComputeResourcesList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByWorkspaceNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.PaginatedComputeResourcesList>;
    listByWorkspaceNext(nextPageLink: string, callback: ServiceCallback<models.PaginatedComputeResourcesList>): void;
    listByWorkspaceNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.PaginatedComputeResourcesList>): void;
}
