/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is regenerated.
 */

import { BaseResource, CloudError } from "ms-rest-azure";
import * as moment from "moment";

export {

  BaseResource,
  CloudError
};

/**
 * ARM resource.
 */
export interface Resource extends BaseResource {
  /**
   * Resource ID.
   */
  readonly id?: string;
  /**
   * Resource name.
   */
  readonly name?: string;
  /**
   * Resource type.
   */
  readonly type?: string;
}

/**
 * Terms properties for provided Publisher/Offer/Plan tuple
 */
export interface AgreementTerms extends Resource {
  /**
   * Publisher identifier string of image being deployed.
   */
  publisher?: string;
  /**
   * Offer identifier string of image being deployed.
   */
  product?: string;
  /**
   * Plan identifier string of image being deployed.
   */
  plan?: string;
  /**
   * Link to HTML with Microsoft and Publisher terms.
   */
  licenseTextLink?: string;
  /**
   * Link to the privacy policy of the publisher.
   */
  privacyPolicyLink?: string;
  /**
   * Date and time in UTC of when the terms were accepted. This is empty if Accepted is false.
   */
  retrieveDatetime?: string;
  /**
   * Terms signature.
   */
  signature?: string;
  /**
   * If any version of the terms have been accepted, otherwise false.
   */
  accepted?: boolean;
}

/**
 * The details of the error.
 */
export interface ErrorResponseError {
  /**
   * Error code.
   */
  readonly code?: string;
  /**
   * Error message indicating why the operation failed.
   */
  readonly message?: string;
}

/**
 * Error response indicates Microsoft.MarketplaceOrdering service is not able to process the
 * incoming request. The reason is provided in the error message.
 */
export interface ErrorResponse {
  /**
   * The details of the error.
   */
  error?: ErrorResponseError;
}

/**
 * The object that represents the operation.
 */
export interface OperationDisplay {
  /**
   * Service provider: Microsoft.MarketplaceOrdering
   */
  provider?: string;
  /**
   * Resource on which the operation is performed: Agreement, virtualmachine, etc.
   */
  resource?: string;
  /**
   * Operation type: Get Agreement, Sign Agreement, Cancel Agreement etc.
   */
  operation?: string;
}

/**
 * Microsoft.MarketplaceOrdering REST API operation
 */
export interface Operation {
  /**
   * Operation name: {provider}/{resource}/{operation}
   */
  name?: string;
  /**
   * The object that represents the operation.
   */
  display?: OperationDisplay;
}

/**
 * Result of the request to list MarketplaceOrdering operations. It contains a list of operations
 * and a URL link to get the next set of results.
 */
export interface OperationListResult extends Array<Operation> {
  /**
   * URL to get the next set of operation list results if there are any.
   */
  readonly nextLink?: string;
}
