/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Project class.
 * @constructor
 * Azure Migrate Project.
 *
 * @member {string} [id] Path reference to this project
 * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Migrate/projects/{projectName}
 * @member {string} [name] Name of the project.
 * @member {string} [type] Type of the object = [Microsoft.Migrate/projects].
 * @member {string} [eTag] For optimistic concurrency control.
 * @member {string} [location] Azure location in which project is created.
 * @member {object} [tags] Tags provided by Azure Tagging service.
 * @member {date} [createdTimestamp] Time when this project was created.
 * Date-Time represented in ISO-8601 format.
 * @member {date} [updatedTimestamp] Time when this project was last updated.
 * Date-Time represented in ISO-8601 format.
 * @member {string} [discoveryStatus] Reports whether project is under
 * discovery. Possible values include: 'Unknown', 'NotStarted', 'InProgress',
 * 'Completed'
 * @member {string} [customerWorkspaceId] ARM ID of the Service Map workspace
 * created by user.
 * @member {string} [customerWorkspaceLocation] Location of the Service Map
 * workspace created by user.
 * @member {date} [lastDiscoveryTimestamp] Time when this project was created.
 * Date-Time represented in ISO-8601 format. This value will be null until
 * discovery is complete.
 * @member {string} [lastDiscoverySessionId] Session id of the last discovery.
 * @member {number} [numberOfGroups] Number of groups created in the project.
 * @member {number} [numberOfMachines] Number of machines in the project.
 * @member {number} [numberOfAssessments] Number of assessments created in the
 * project.
 * @member {date} [lastAssessmentTimestamp] Time when last assessment was
 * created. Date-Time represented in ISO-8601 format. This value will be null
 * until assessment is created.
 * @member {string} [provisioningState] Provisioning state of the project.
 * Possible values include: 'Accepted', 'Creating', 'Deleting', 'Failed',
 * 'Moving', 'Succeeded'
 */
export interface Project extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  eTag?: string;
  location?: string;
  tags?: any;
  readonly createdTimestamp?: Date;
  readonly updatedTimestamp?: Date;
  readonly discoveryStatus?: string;
  customerWorkspaceId?: string;
  customerWorkspaceLocation?: string;
  readonly lastDiscoveryTimestamp?: Date;
  readonly lastDiscoverySessionId?: string;
  readonly numberOfGroups?: number;
  readonly numberOfMachines?: number;
  readonly numberOfAssessments?: number;
  readonly lastAssessmentTimestamp?: Date;
  provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the Group class.
 * @constructor
 * A group created in a Migration project.
 *
 * @member {string} [id] Path reference to this group.
 * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Migrate/projects/{projectName}/groups/{groupName}
 * @member {string} [name] Name of the group.
 * @member {string} [eTag] For optimistic concurrency control.
 * @member {string} [type] Type of the object =
 * [Microsoft.Migrate/projects/groups].
 * @member {array} machines List of machine names that are part of this group.
 * @member {array} [assessments] List of References to Assessments created on
 * this group.
 * @member {date} [createdTimestamp] Time when this project was created.
 * Date-Time represented in ISO-8601 format.
 * @member {date} [updatedTimestamp] Time when this project was last updated.
 * Date-Time represented in ISO-8601 format.
 */
export interface Group extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  eTag?: string;
  readonly type?: string;
  machines: string[];
  readonly assessments?: string[];
  readonly createdTimestamp?: Date;
  readonly updatedTimestamp?: Date;
}

/**
 * @class
 * Initializes a new instance of the Assessment class.
 * @constructor
 * An assessment created for a group in the Migration project.
 *
 * @member {string} [id] Path reference to this assessment.
 * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Migrate/projects/{projectName}/groups/{groupName}/assessment/{assessmentName}
 * @member {string} [name] Unique name of an assessment.
 * @member {string} [eTag] For optimistic concurrency control.
 * @member {string} [type] Type of the object =
 * [Microsoft.Migrate/projects/groups/assessments].
 * @member {string} azureLocation Target Azure location for which the machines
 * should be assessed. These enums are the same as used by Compute API.
 * Possible values include: 'Unknown', 'EastAsia', 'SoutheastAsia',
 * 'AustraliaEast', 'AustraliaSoutheast', 'BrazilSouth', 'CanadaCentral',
 * 'CanadaEast', 'WestEurope', 'NorthEurope', 'CentralIndia', 'SouthIndia',
 * 'WestIndia', 'JapanEast', 'JapanWest', 'KoreaCentral', 'KoreaSouth',
 * 'UkWest', 'UkSouth', 'NorthCentralUs', 'EastUs', 'WestUs2',
 * 'SouthCentralUs', 'CentralUs', 'EastUs2', 'WestUs', 'WestCentralUs',
 * 'GermanyCentral', 'GermanyNortheast', 'ChinaNorth', 'ChinaEast'
 * @member {string} azureOfferCode Offer code according to which cost
 * estimation is done. Possible values include: 'Unknown', 'MSAZR0003P',
 * 'MSAZR0044P', 'MSAZR0059P', 'MSAZR0060P', 'MSAZR0062P', 'MSAZR0063P',
 * 'MSAZR0064P', 'MSAZR0029P', 'MSAZR0022P', 'MSAZR0023P', 'MSAZR0148P',
 * 'MSAZR0025P', 'MSAZR0036P', 'MSAZR0120P', 'MSAZR0121P', 'MSAZR0122P',
 * 'MSAZR0123P', 'MSAZR0124P', 'MSAZR0125P', 'MSAZR0126P', 'MSAZR0127P',
 * 'MSAZR0128P', 'MSAZR0129P', 'MSAZR0130P', 'MSAZR0111P', 'MSAZR0144P',
 * 'MSAZR0149P', 'MSMCAZR0044P', 'MSMCAZR0059P', 'MSMCAZR0060P',
 * 'MSMCAZR0063P', 'MSMCAZR0120P', 'MSMCAZR0121P', 'MSMCAZR0125P',
 * 'MSMCAZR0128P', 'MSAZRDE0003P', 'MSAZRDE0044P'
 * @member {string} azurePricingTier Pricing tier for Size evaluation. Possible
 * values include: 'Standard', 'Basic'
 * @member {string} azureStorageRedundancy Storage Redundancy type offered by
 * Azure. Possible values include: 'Unknown', 'LocallyRedundant',
 * 'ZoneRedundant', 'GeoRedundant', 'ReadAccessGeoRedundant'
 * @member {number} scalingFactor Scaling factor used over utilization data to
 * add a performance buffer for new machines to be created in Azure. Min Value
 * = 1.0, Max value = 1.9, Default = 1.3.
 * @member {string} percentile Percentile of performance data used to recommend
 * Azure size. Possible values include: 'Percentile50', 'Percentile90',
 * 'Percentile95', 'Percentile99'
 * @member {string} timeRange Time range of performance data used to recommend
 * a size. Possible values include: 'Day', 'Week', 'Month'
 * @member {string} stage User configurable setting that describes the status
 * of the assessment. Possible values include: 'InProgress', 'UnderReview',
 * 'Approved'
 * @member {string} currency Currency to report prices in. Possible values
 * include: 'Unknown', 'USD', 'DKK', 'CAD', 'IDR', 'JPY', 'KRW', 'NZD', 'NOK',
 * 'RUB', 'SAR', 'ZAR', 'SEK', 'TRY', 'GBP', 'MXN', 'MYR', 'INR', 'HKD', 'BRL',
 * 'TWD', 'EUR', 'CHF', 'ARS', 'AUD', 'CNY'
 * @member {string} azureHybridUseBenefit AHUB discount on windows virtual
 * machines. Possible values include: 'Unknown', 'Yes', 'No'
 * @member {number} discountPercentage Custom discount percentage to be applied
 * on final costs. Can be in the range [0, 100].
 * @member {number} [confidenceRatingInPercentage] Confidence rating percentage
 * for assessment. Can be in the range [0, 100].
 * @member {string} sizingCriterion Assessment sizing criterion. Possible
 * values include: 'PerformanceBased', 'AsOnPremises'
 * @member {date} [pricesTimestamp] Time when the Azure Prices were queried.
 * Date-Time represented in ISO-8601 format.
 * @member {date} [createdTimestamp] Time when this project was created.
 * Date-Time represented in ISO-8601 format.
 * @member {date} [updatedTimestamp] Time when this project was last updated.
 * Date-Time represented in ISO-8601 format.
 * @member {number} [monthlyComputeCost] Monthly compute cost estimate for the
 * machines that are part of this assessment as a group, for a 31-day month.
 * @member {number} [monthlyBandwidthCost] Monthly network cost estimate for
 * the machines that are part of this assessment as a group, for a 31-day
 * month.
 * @member {number} [monthlyStorageCost] Monthly storage cost estimate for the
 * machines that are part of this assessment as a group, for a 31-day month.
 * @member {string} [status] Wheter the assessment has been created and is
 * valid. Possible values include: 'Created', 'Updated', 'Running',
 * 'Completed', 'Invalid'
 * @member {number} [numberOfMachines] Number of assessed machines part of this
 * assessment.
 */
export interface Assessment extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  eTag?: string;
  readonly type?: string;
  azureLocation: string;
  azureOfferCode: string;
  azurePricingTier: string;
  azureStorageRedundancy: string;
  scalingFactor: number;
  percentile: string;
  timeRange: string;
  stage: string;
  currency: string;
  azureHybridUseBenefit: string;
  discountPercentage: number;
  readonly confidenceRatingInPercentage?: number;
  sizingCriterion: string;
  readonly pricesTimestamp?: Date;
  readonly createdTimestamp?: Date;
  readonly updatedTimestamp?: Date;
  readonly monthlyComputeCost?: number;
  readonly monthlyBandwidthCost?: number;
  readonly monthlyStorageCost?: number;
  readonly status?: string;
  readonly numberOfMachines?: number;
}

/**
 * @class
 * Initializes a new instance of the Disk class.
 * @constructor
 * A disk discovered on a machine.
 *
 * @member {number} [gigabytesAllocated] Gigabytes of storage provisioned for
 * this disk.
 * @member {number} [gigabytesConsumed] Gigabytes of storage consumed by this
 * disk.
 */
export interface Disk {
  readonly gigabytesAllocated?: number;
  readonly gigabytesConsumed?: number;
}

/**
 * @class
 * Initializes a new instance of the NetworkAdapter class.
 * @constructor
 * A network adapter discovered on a machine.
 *
 * @member {string} [macAddress] MAC Address of the network adapter.
 * @member {array} [ipAddresses] List of IP Addresses on the network adapter.
 */
export interface NetworkAdapter {
  readonly macAddress?: string;
  readonly ipAddresses?: string[];
}

/**
 * @class
 * Initializes a new instance of the Machine class.
 * @constructor
 * A machine in a migration project.
 *
 * @member {string} [id] Path reference to this machine.
 * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Migrate/projects/{projectName}/machines/{machineName}
 * @member {string} [name] Name of the machine. It is a GUID which is unique
 * identifier of machine in private data center. For user-readable name, we
 * have a displayName property on this machine.
 * @member {string} [eTag] For optimistic concurrency control.
 * @member {string} [type] Type of the object =
 * [Microsoft.Migrate/projects/machines].
 * @member {string} [bootType] Boot type of the machine. Possible values
 * include: 'Unknown', 'EFI', 'BIOS'
 * @member {string} [datacenterContainer] Container defined in the management
 * solution that this machine is part of in the datacenter.
 * @member {string} [datacenterManagementServer] Name of the server hosting the
 * datacenter management solution.
 * @member {string} [datacenterMachineId] ID of the machine as tracked by the
 * datacenter management solution.
 * @member {string} [datacenterManagementServerId] ID of the server hosting the
 * datacenter management solution.
 * @member {string} [description] Description of the machine
 * @member {string} [displayName] User readable name of the machine as defined
 * by the user in their private datacenter.
 * @member {number} [megabytesOfMemory] Memory in Megabytes.
 * @member {number} [numberOfCores] Processor count.
 * @member {string} [operatingSystem] Operating System of the machine.
 * @member {array} [groups] List of references to the groups that the machine
 * is member of.
 * @member {date} [createdTimestamp] Time when this machine was created.
 * Date-Time represented in ISO-8601 format.
 * @member {date} [updatedTimestamp] Time when this machine was last updated.
 * Date-Time represented in ISO-8601 format.
 * @member {date} [discoveredTimestamp] Time when this machine was discovered
 * by Azure Migrate agent. Date-Time represented in ISO-8601 format.
 * @member {object} [disks] Dictionary of disks attached to the machine. Key is
 * ID of disk. Value is a disk object
 * @member {object} [networkAdapters] Dictionary of network adapters attached
 * to the machine. Key is ID of network adapter. Value is a network adapter
 * object
 */
export interface Machine extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  eTag?: string;
  readonly type?: string;
  readonly bootType?: string;
  readonly datacenterContainer?: string;
  readonly datacenterManagementServer?: string;
  readonly datacenterMachineId?: string;
  readonly datacenterManagementServerId?: string;
  readonly description?: string;
  readonly displayName?: string;
  readonly megabytesOfMemory?: number;
  readonly numberOfCores?: number;
  readonly operatingSystem?: string;
  readonly groups?: string[];
  readonly createdTimestamp?: Date;
  readonly updatedTimestamp?: Date;
  readonly discoveredTimestamp?: Date;
  readonly disks?: { [propertyName: string]: Disk };
  readonly networkAdapters?: { [propertyName: string]: NetworkAdapter };
}

/**
 * @class
 * Initializes a new instance of the AssessedDisk class.
 * @constructor
 * A disk assessed for an assessment.
 *
 * @member {string} [name] Name of the assessed disk.
 * @member {number} [gigabytesProvisioned] Gigabytes of storage provisioned for
 * this disk.
 * @member {number} [gigabytesConsumed] Gigabytes of storage consumed by this
 * disk.
 * @member {number} [megabytesPerSecondOfRead] Disk throughput in MegaBytes per
 * second.
 * @member {number} [megabytesPerSecondOfReadDataPointsExpected] Expected data
 * points for MegaBytes per second of read.
 * @member {number} [megabytesPerSecondOfReadDataPointsReceived] Received data
 * points for MegaBytes per second of read.
 * @member {number} [megabytesPerSecondOfWrite] Disk throughput in MegaBytes
 * per second.
 * @member {number} [megabytesPerSecondOfWriteDataPointsExpected] Expected data
 * points for MegaBytes per second of write.
 * @member {number} [megabytesPerSecondOfWriteDataPointsReceived] Received data
 * points for MegaBytes per second of write.
 * @member {number} [numberOfReadOperationsPerSecond] Number of read operations
 * per second for the disk.
 * @member {number} [numberOfReadOperationsPerSecondDataPointsExpected]
 * Expected number of data points for read operations per second.
 * @member {number} [numberOfReadOperationsPerSecondDataPointsReceived]
 * Received number of data points for read operations per second.
 * @member {number} [numberOfWriteOperationsPerSecond] Number of read and write
 * operations per second for the disk.
 * @member {number} [numberOfWriteOperationsPerSecondDataPointsExpected]
 * Expected number of data points for write operations per second.
 * @member {number} [numberOfWriteOperationsPerSecondDataPointsReceived]
 * Received number of data points for write operations per second.
 * @member {number} [monthlyStorageCost] Estimated aggregate storage cost for a
 * 31-day month for this disk.
 * @member {string} [recommendedDiskType] Storage type selected for this disk.
 * Possible values include: 'Unknown', 'Standard', 'Premium'
 * @member {string} [recommendedDiskSize] Recommended Azure size for the disk,
 * given utilization data and preferences set on Assessment. Possible values
 * include: 'Unknown', 'Standard_S4', 'Standard_S6', 'Standard_S10',
 * 'Standard_S20', 'Standard_S30', 'Standard_S40', 'Standard_S50',
 * 'Premium_P4', 'Premium_P6', 'Premium_P10', 'Premium_P20', 'Premium_P30',
 * 'Premium_P40', 'Premium_P50'
 * @member {number} [gigabytesForRecommendedDiskSize] Gigabytes of storage
 * provided by the recommended Azure disk size.
 * @member {string} [suitability] Whether this disk is suitable for Azure.
 * Possible values include: 'Unknown', 'NotSuitable', 'Suitable',
 * 'ConditionallySuitable', 'ReadinessUnknown'
 * @member {string} [suitabilityExplanation] If disk is suitable, this explains
 * the reasons and mitigation steps. Possible values include: 'Unknown',
 * 'NotApplicable', 'DiskSizeGreaterThanSupported',
 * 'NoSuitableDiskSizeForIops', 'NoSuitableDiskSizeForThroughput',
 * 'NoDiskSizeFoundInSelectedLocation', 'NoDiskSizeFoundForSelectedRedundancy',
 * 'InternalErrorOccurredForDiskEvaluation'
 */
export interface AssessedDisk {
  readonly name?: string;
  readonly gigabytesProvisioned?: number;
  readonly gigabytesConsumed?: number;
  readonly megabytesPerSecondOfRead?: number;
  readonly megabytesPerSecondOfReadDataPointsExpected?: number;
  readonly megabytesPerSecondOfReadDataPointsReceived?: number;
  readonly megabytesPerSecondOfWrite?: number;
  readonly megabytesPerSecondOfWriteDataPointsExpected?: number;
  readonly megabytesPerSecondOfWriteDataPointsReceived?: number;
  readonly numberOfReadOperationsPerSecond?: number;
  readonly numberOfReadOperationsPerSecondDataPointsExpected?: number;
  readonly numberOfReadOperationsPerSecondDataPointsReceived?: number;
  readonly numberOfWriteOperationsPerSecond?: number;
  readonly numberOfWriteOperationsPerSecondDataPointsExpected?: number;
  readonly numberOfWriteOperationsPerSecondDataPointsReceived?: number;
  readonly monthlyStorageCost?: number;
  readonly recommendedDiskType?: string;
  readonly recommendedDiskSize?: string;
  readonly gigabytesForRecommendedDiskSize?: number;
  readonly suitability?: string;
  readonly suitabilityExplanation?: string;
}

/**
 * @class
 * Initializes a new instance of the AssessedNetworkAdapter class.
 * @constructor
 * A network adapter assessed for an assessment.
 *
 * @member {string} [macAddress] MAC Address of the network adapter.
 * @member {array} [ipAddresses] List of IP Addresses on the network adapter.
 * @member {number} [monthlyBandwidthCosts] Monthly cost estimate for network
 * bandwidth used by this network adapter.
 * @member {number} [megabytesPerSecondReceived] Adapter throughput for
 * incoming traffic in MegaBytes per second.
 * @member {number} [megabytesPerSecondReceivedDataPointsExpected] Expected
 * data points for incoming traffic in MegaBytes per second.
 * @member {number} [megabytesPerSecondOfReadDataPointsReceived] Received data
 * points for incoming traffic in MegaBytes per second.
 * @member {number} [megabytesPerSecondTransmitted] Adapter throughput for
 * outgoing traffic in MegaBytes per second.
 * @member {number} [megabytesPerSecondTransmittedDataPointsExpected] Expected
 * data points for outgoing traffic in MegaBytes per second.
 * @member {number} [megabytesPerSecondTransmittedDataPointsReceived] Received
 * data points for outgoing traffic in MegaBytes per second.
 * @member {number} [netGigabytesTransmittedPerMonth] Gigabytes transmitted
 * through this adapter each month.
 * @member {string} [suitability] Whether this adapter is suitable for Azure.
 * Possible values include: 'Unknown', 'NotSuitable', 'Suitable',
 * 'ConditionallySuitable', 'ReadinessUnknown'
 * @member {string} [suitabilityExplanation] If network adapter is suitable,
 * this explains the reasons and mitigation steps. Possible values include:
 * 'Unknown', 'NotApplicable', 'InternalErrorOccured'
 */
export interface AssessedNetworkAdapter {
  readonly macAddress?: string;
  readonly ipAddresses?: string[];
  readonly monthlyBandwidthCosts?: number;
  readonly megabytesPerSecondReceived?: number;
  readonly megabytesPerSecondReceivedDataPointsExpected?: number;
  readonly megabytesPerSecondOfReadDataPointsReceived?: number;
  readonly megabytesPerSecondTransmitted?: number;
  readonly megabytesPerSecondTransmittedDataPointsExpected?: number;
  readonly megabytesPerSecondTransmittedDataPointsReceived?: number;
  netGigabytesTransmittedPerMonth?: number;
  readonly suitability?: string;
  readonly suitabilityExplanation?: string;
}

/**
 * @class
 * Initializes a new instance of the AssessedMachine class.
 * @constructor
 * A machine evaluated as part of an assessment.
 *
 * @member {string} [id] Path reference to this assessed machine.
 * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Migrate/projects/{projectName}/groups/{groupName}/assessments/{assessmentName}/assessedMachines/{assessedMachineName}
 * @member {string} [name] Name of the machine.
 * @member {string} [eTag] For optimistic concurrency control.
 * @member {string} [type] Type of the object =
 * [Microsoft.Migrate/projects/groups/assessments/assessedMachines].
 * @member {array} [groups] List of references to the groups that the machine
 * is member of.
 * @member {date} [discoveredTimestamp] Time when this machine was discovered
 * by Azure Migrate agent. Date-Time represented in ISO-8601 format.
 * @member {string} [bootType] Boot type of the machine. Possible values
 * include: 'Unknown', 'EFI', 'BIOS'
 * @member {string} [datacenterContainer] Container defined in the management
 * solution that this machine is part of in the datacenter.
 * @member {string} [datacenterManagementServer] Name of the server hosting the
 * datacenter management solution.
 * @member {string} [datacenterMachineId] ID of the machine as tracked by the
 * datacenter management solution.
 * @member {string} [datacenterManagementServerId] ID of the server hosting the
 * datacenter management solution.
 * @member {string} [description] Description of the machine
 * @member {string} [displayName] User readable name of the machine as defined
 * by the user in their private datacenter.
 * @member {number} [megabytesOfMemory] Memory in Megabytes.
 * @member {number} [numberOfCores] Processor count.
 * @member {string} [operatingSystem] Operating System of the machine.
 * @member {number} [monthlyBandwidthCost] Monthly network cost estimate for
 * the network adapters that are attached to this machine as a group, for a
 * 31-day month.
 * @member {number} [monthlyStorageCost] Monthly storage cost estimate for the
 * disks that are attached to this machine as a group, for a 31-day month.
 * @member {object} [disks] Dictionary of disks attached to the machine. Key is
 * ID of disk. Value is a disk object.
 * @member {object} [networkAdapters] Dictionary of network adapters attached
 * to the machine. Key is name of the adapter. Value is a network adapter
 * object.
 * @member {string} [recommendedSize] Recommended Azure size for this machine.
 * Possible values include: 'Unknown', 'Basic_A0', 'Basic_A1', 'Basic_A2',
 * 'Basic_A3', 'Basic_A4', 'Standard_A0', 'Standard_A1', 'Standard_A2',
 * 'Standard_A3', 'Standard_A4', 'Standard_A5', 'Standard_A6', 'Standard_A7',
 * 'Standard_A8', 'Standard_A9', 'Standard_A10', 'Standard_A11',
 * 'Standard_A1_v2', 'Standard_A2_v2', 'Standard_A4_v2', 'Standard_A8_v2',
 * 'Standard_A2m_v2', 'Standard_A4m_v2', 'Standard_A8m_v2', 'Standard_D1',
 * 'Standard_D2', 'Standard_D3', 'Standard_D4', 'Standard_D11', 'Standard_D12',
 * 'Standard_D13', 'Standard_D14', 'Standard_D1_v2', 'Standard_D2_v2',
 * 'Standard_D3_v2', 'Standard_D4_v2', 'Standard_D5_v2', 'Standard_D11_v2',
 * 'Standard_D12_v2', 'Standard_D13_v2', 'Standard_D14_v2', 'Standard_D15_v2',
 * 'Standard_DS1', 'Standard_DS2', 'Standard_DS3', 'Standard_DS4',
 * 'Standard_DS11', 'Standard_DS12', 'Standard_DS13', 'Standard_DS14',
 * 'Standard_DS1_v2', 'Standard_DS2_v2', 'Standard_DS3_v2', 'Standard_DS4_v2',
 * 'Standard_DS5_v2', 'Standard_DS11_v2', 'Standard_DS12_v2',
 * 'Standard_DS13_v2', 'Standard_DS14_v2', 'Standard_DS15_v2', 'Standard_F1',
 * 'Standard_F2', 'Standard_F4', 'Standard_F8', 'Standard_F16', 'Standard_F1s',
 * 'Standard_F2s', 'Standard_F4s', 'Standard_F8s', 'Standard_F16s',
 * 'Standard_G1', 'Standard_G2', 'Standard_G3', 'Standard_G4', 'Standard_G5',
 * 'Standard_GS1', 'Standard_GS2', 'Standard_GS3', 'Standard_GS4',
 * 'Standard_GS5', 'Standard_H8', 'Standard_H16', 'Standard_H8m',
 * 'Standard_H16m', 'Standard_H16r', 'Standard_H16mr', 'Standard_L4s',
 * 'Standard_L8s', 'Standard_L16s', 'Standard_L32s'
 * @member {number} [numberOfCoresForRecommendedSize] Number of CPU cores in
 * the Recommended Azure VM Size.
 * @member {number} [megabytesOfMemoryForRecommendedSize] Megabytes of memory
 * in the Recommended Azure VM Size.
 * @member {number} [monthlyComputeCostForRecommendedSize] Compute Cost for a
 * 31-day month, if the machine is migrated to Azure with the Recommended Size.
 * @member {number} [percentageCoresUtilization] Utilization percentage of the
 * processor core as observed in the private data center, in the Time Range
 * selected on Assessment, reported as the Percentile value based on the
 * percentile number selected in assessment.
 * @member {number} [percentageMemoryUtilization] Utilization percentage of the
 * memory as observed in the private data center, in the Time Range selected on
 * Assessment, reported as the Percentile value based on the percentile number
 * selected in assessment.
 * @member {number} [percentageCoresUtilizationDataPointsExpected] Expected
 * data points for percentage of cores utilization.
 * @member {number} [percentageCoresUtilizationDataPointsReceived] Received
 * data points for percentage of cores utilization.
 * @member {number} [percentageMemoryUtilizationDataPointsExpected] Expected
 * data points for percentage of memory utilization.
 * @member {number} [percentageMemoryUtilizationDataPointsReceived] Received
 * data points for percentage of memory utilization.
 * @member {string} [suitability] Whether machine is suitable for migration to
 * Azure. Possible values include: 'Unknown', 'NotSuitable', 'Suitable',
 * 'ConditionallySuitable', 'ReadinessUnknown'
 * @member {string} [suitabilityExplanation] If machine is not ready to be
 * migrated, this explains the reasons and mitigation steps. Possible values
 * include: 'Unknown', 'NotApplicable',
 * 'GuestOperatingSystemArchitectureNotSupported',
 * 'GuestOperatingSystemNotSupported', 'BootTypeNotSupported',
 * 'MoreDisksThanSupported', 'NoSuitableVmSizeFound',
 * 'OneOrMoreDisksNotSuitable', 'OneOrMoreAdaptersNotSuitable',
 * 'InternalErrorOccuredDuringComputeEvaluation',
 * 'InternalErrorOccuredDuringStorageEvaluation',
 * 'InternalErrorOccuredDuringNetworkEvaluation',
 * 'NoVmSizeSupportsStoragePerformance', 'NoVmSizeSupportsNetworkPerformance',
 * 'NoVmSizeForSelectedPricingTier', 'NoVmSizeForSelectedAzureLocation',
 * 'CheckRedHatLinuxVersion', 'CheckOpenSuseLinuxVersion',
 * 'CheckWindowsServer2008R2Version', 'CheckCentOsVersion',
 * 'CheckDebianLinuxVersion', 'CheckSuseLinuxVersion',
 * 'CheckOracleLinuxVersion', 'CheckUbuntuLinuxVersion',
 * 'CheckCoreOsLinuxVersion', 'WindowsServerVersionConditionallySupported',
 * 'NoGuestOperatingSystemConditionallySupported',
 * 'WindowsClientVersionsConditionallySupported', 'BootTypeUnknown',
 * 'GuestOperatingSystemUnknown', 'WindowsServerVersionsSupportedWithCaveat',
 * 'WindowsOSNoLongerUnderMSSupport',
 * 'EndorsedWithConditionsLinuxDistributions', 'UnendorsedLinuxDistributions',
 * 'NoVmSizeForStandardPricingTier', 'NoVmSizeForBasicPricingTier'
 * @member {date} [createdTimestamp] Time when this machine was created.
 * Date-Time represented in ISO-8601 format.
 * @member {date} [updatedTimestamp] Time when this machine was last updated.
 * Date-Time represented in ISO-8601 format.
 */
export interface AssessedMachine extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  eTag?: string;
  readonly type?: string;
  readonly groups?: string[];
  readonly discoveredTimestamp?: Date;
  readonly bootType?: string;
  readonly datacenterContainer?: string;
  readonly datacenterManagementServer?: string;
  readonly datacenterMachineId?: string;
  readonly datacenterManagementServerId?: string;
  readonly description?: string;
  readonly displayName?: string;
  readonly megabytesOfMemory?: number;
  readonly numberOfCores?: number;
  readonly operatingSystem?: string;
  readonly monthlyBandwidthCost?: number;
  readonly monthlyStorageCost?: number;
  readonly disks?: { [propertyName: string]: AssessedDisk };
  readonly networkAdapters?: { [propertyName: string]: AssessedNetworkAdapter };
  readonly recommendedSize?: string;
  readonly numberOfCoresForRecommendedSize?: number;
  readonly megabytesOfMemoryForRecommendedSize?: number;
  readonly monthlyComputeCostForRecommendedSize?: number;
  readonly percentageCoresUtilization?: number;
  readonly percentageMemoryUtilization?: number;
  readonly percentageCoresUtilizationDataPointsExpected?: number;
  readonly percentageCoresUtilizationDataPointsReceived?: number;
  readonly percentageMemoryUtilizationDataPointsExpected?: number;
  readonly percentageMemoryUtilizationDataPointsReceived?: number;
  readonly suitability?: string;
  readonly suitabilityExplanation?: string;
  readonly createdTimestamp?: Date;
  readonly updatedTimestamp?: Date;
}

/**
 * @class
 * Initializes a new instance of the ProjectKey class.
 * @constructor
 * ID and Key for Migration Project.
 *
 * @member {string} [workspaceId] ID of Migration Project.
 * @member {string} [workspaceKey] Key of Migration Project.
 */
export interface ProjectKey extends BaseResource {
  readonly workspaceId?: string;
  readonly workspaceKey?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Displayable properties of the operation.
 *
 * @member {string} [provider] Provider of the operation.
 * @member {string} [resource] Resource operated on by the operation.
 * @member {string} [operation] Operation Type.
 * @member {string} [description] Description of the operation.
 */
export interface OperationDisplay {
  readonly provider?: string;
  readonly resource?: string;
  readonly operation?: string;
  readonly description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * A REST API operation supported by the provider.
 *
 * @member {string} [name] Name of the operation.
 * @member {object} [display] Displayable properties of the operation.
 * @member {string} [display.provider] Provider of the operation.
 * @member {string} [display.resource] Resource operated on by the operation.
 * @member {string} [display.operation] Operation Type.
 * @member {string} [display.description] Description of the operation.
 * @member {string} [origin] Origin of the operation.
 */
export interface Operation {
  readonly name?: string;
  display?: OperationDisplay;
  readonly origin?: string;
}

/**
 * @class
 * Initializes a new instance of the DownloadUrl class.
 * @constructor
 * Download URL for assessment report.
 *
 * @member {string} [assessmentReportUrl] Hyperlink to download report.
 * @member {date} [expirationTime] Expiry date of download url.
 */
export interface DownloadUrl {
  readonly assessmentReportUrl?: string;
  readonly expirationTime?: Date;
}


/**
 * @class
 * Initializes a new instance of the ProjectResultList class.
 * @constructor
 * List of projects.
 *
 */
export interface ProjectResultList extends Array<Project> {
}

/**
 * @class
 * Initializes a new instance of the MachineResultList class.
 * @constructor
 * List of machines.
 *
 */
export interface MachineResultList extends Array<Machine> {
}

/**
 * @class
 * Initializes a new instance of the GroupResultList class.
 * @constructor
 * List of groups.
 *
 */
export interface GroupResultList extends Array<Group> {
}

/**
 * @class
 * Initializes a new instance of the AssessmentResultList class.
 * @constructor
 * List of assessments.
 *
 */
export interface AssessmentResultList extends Array<Assessment> {
}

/**
 * @class
 * Initializes a new instance of the AssessedMachineResultList class.
 * @constructor
 * List of assessed machines.
 *
 */
export interface AssessedMachineResultList extends Array<AssessedMachine> {
}

/**
 * @class
 * Initializes a new instance of the OperationResultList class.
 * @constructor
 * List of API operations.
 *
 */
export interface OperationResultList extends Array<Operation> {
}
