/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Display metadata associated with the operation.
 *
 * @member {string} [provider] Service provider: Microsoft
 * OperationsManagement.
 * @member {string} [resource] Resource on which the operation is performed
 * etc.
 * @member {string} [operation] Type of operation: get, read, delete, etc.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Supported operation of OperationalInsights resource provider.
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {object} [display] Display metadata associated with the operation.
 * @member {string} [display.provider] Service provider: Microsoft
 * OperationsManagement.
 * @member {string} [display.resource] Resource on which the operation is
 * performed etc.
 * @member {string} [display.operation] Type of operation: get, read, delete,
 * etc.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the LinkedService class.
 * @constructor
 * The top level Linked service resource container.
 *
 * @member {string} resourceId The resource id of the resource that will be
 * linked to the workspace.
 */
export interface LinkedService extends BaseResource {
  resourceId: string;
}

/**
 * @class
 * Initializes a new instance of the DataSource class.
 * @constructor
 * Datasources under OMS Workspace.
 *
 * @member {object} properties The data source properties in raw json format,
 * each kind of data source have it's own schema.
 * @member {string} [eTag] The ETag of the data source.
 * @member {string} kind Possible values include: 'AzureActivityLog',
 * 'ChangeTrackingPath', 'ChangeTrackingDefaultPath',
 * 'ChangeTrackingDefaultRegistry', 'ChangeTrackingCustomRegistry',
 * 'CustomLog', 'CustomLogCollection', 'GenericDataSource', 'IISLogs',
 * 'LinuxPerformanceObject', 'LinuxPerformanceCollection', 'LinuxSyslog',
 * 'LinuxSyslogCollection', 'WindowsEvent', 'WindowsPerformanceCounter'
 */
export interface DataSource extends BaseResource {
  properties: any;
  eTag?: string;
  kind: string;
}

/**
 * @class
 * Initializes a new instance of the DataSourceFilter class.
 * @constructor
 * DataSource filter. Right now, only filter by kind is supported.
 *
 * @member {string} [kind] Possible values include: 'AzureActivityLog',
 * 'ChangeTrackingPath', 'ChangeTrackingDefaultPath',
 * 'ChangeTrackingDefaultRegistry', 'ChangeTrackingCustomRegistry',
 * 'CustomLog', 'CustomLogCollection', 'GenericDataSource', 'IISLogs',
 * 'LinuxPerformanceObject', 'LinuxPerformanceCollection', 'LinuxSyslog',
 * 'LinuxSyslogCollection', 'WindowsEvent', 'WindowsPerformanceCounter'
 */
export interface DataSourceFilter {
  kind?: string;
}

/**
 * @class
 * Initializes a new instance of the IntelligencePack class.
 * @constructor
 * Intelligence Pack containing a string name and boolean indicating if it's
 * enabled.
 *
 * @member {string} [name] The name of the intelligence pack.
 * @member {boolean} [enabled] The enabled boolean for the intelligence pack.
 * @member {string} [displayName] The display name of the intelligence pack.
 */
export interface IntelligencePack {
  name?: string;
  enabled?: boolean;
  displayName?: string;
}

/**
 * @class
 * Initializes a new instance of the SharedKeys class.
 * @constructor
 * The shared keys for a workspace.
 *
 * @member {string} [primarySharedKey] The primary shared key of a workspace.
 * @member {string} [secondarySharedKey] The secondary shared key of a
 * workspace.
 */
export interface SharedKeys {
  primarySharedKey?: string;
  secondarySharedKey?: string;
}

/**
 * @class
 * Initializes a new instance of the MetricName class.
 * @constructor
 * The name of a metric.
 *
 * @member {string} [value] The system name of the metric.
 * @member {string} [localizedValue] The localized name of the metric.
 */
export interface MetricName {
  value?: string;
  localizedValue?: string;
}

/**
 * @class
 * Initializes a new instance of the UsageMetric class.
 * @constructor
 * A metric describing the usage of a resource.
 *
 * @member {object} [name] The name of the metric.
 * @member {string} [name.value] The system name of the metric.
 * @member {string} [name.localizedValue] The localized name of the metric.
 * @member {string} [unit] The units used for the metric.
 * @member {number} [currentValue] The current value of the metric.
 * @member {number} [limit] The quota limit for the metric.
 * @member {date} [nextResetTime] The time that the metric's value will reset.
 * @member {string} [quotaPeriod] The quota period that determines the length
 * of time between value resets.
 */
export interface UsageMetric {
  name?: MetricName;
  unit?: string;
  currentValue?: number;
  limit?: number;
  nextResetTime?: Date;
  quotaPeriod?: string;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroup class.
 * @constructor
 * A management group that is connected to a workspace
 *
 * @member {number} [serverCount] The number of servers connected to the
 * management group.
 * @member {boolean} [isGateway] Gets or sets a value indicating whether the
 * management group is a gateway.
 * @member {string} [name] The name of the management group.
 * @member {string} [id] The unique ID of the management group.
 * @member {date} [created] The datetime that the management group was created.
 * @member {date} [dataReceived] The last datetime that the management group
 * received data.
 * @member {string} [version] The version of System Center that is managing the
 * management group.
 * @member {string} [sku] The SKU of System Center that is managing the
 * management group.
 */
export interface ManagementGroup {
  serverCount?: number;
  isGateway?: boolean;
  name?: string;
  id?: string;
  created?: Date;
  dataReceived?: Date;
  version?: string;
  sku?: string;
}

/**
 * @class
 * Initializes a new instance of the Sku class.
 * @constructor
 * The SKU (tier) of a workspace.
 *
 * @member {string} name The name of the SKU. Possible values include: 'Free',
 * 'Standard', 'Premium', 'Unlimited', 'PerNode', 'PerGB2018', 'Standalone'
 */
export interface Sku {
  name: string;
}

/**
 * @class
 * Initializes a new instance of the Workspace class.
 * @constructor
 * The top level Workspace resource container.
 *
 * @member {string} [provisioningState] The provisioning state of the
 * workspace. Possible values include: 'Creating', 'Succeeded', 'Failed',
 * 'Canceled', 'Deleting', 'ProvisioningAccount'
 * @member {string} [source] The source of the workspace.  Source defines where
 * the workspace was created. 'Azure' implies it was created in Azure.
 * 'External' implies it was created via the Operational Insights Portal. This
 * value is set on the service side and read-only on the client side.
 * @member {string} [customerId] The ID associated with the workspace.  Setting
 * this value at creation time allows the workspace being created to be linked
 * to an existing workspace.
 * @member {string} [portalUrl] The URL of the Operational Insights portal for
 * this workspace.  This value is set on the service side and read-only on the
 * client side.
 * @member {object} [sku] The SKU of the workspace.
 * @member {string} [sku.name] The name of the SKU. Possible values include:
 * 'Free', 'Standard', 'Premium', 'Unlimited', 'PerNode', 'PerGB2018',
 * 'Standalone'
 * @member {number} [retentionInDays] The workspace data retention in days. -1
 * means Unlimited retention for the Unlimited Sku. 730 days is the maximum
 * allowed for all other Skus.
 * @member {string} [eTag] The ETag of the workspace.
 */
export interface Workspace extends BaseResource {
  provisioningState?: string;
  source?: string;
  customerId?: string;
  portalUrl?: string;
  sku?: Sku;
  retentionInDays?: number;
  eTag?: string;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * The resource definition.
 *
 * @member {string} [id] Resource Id
 * @member {string} [name] Resource name
 * @member {string} [type] Resource type
 * @member {string} [location] Resource location
 * @member {object} [tags] Resource tags
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ProxyResource class.
 * @constructor
 * Common properties of proxy resource.
 *
 * @member {string} [id] Resource ID.
 * @member {string} [name] Resource name.
 * @member {string} [type] Resource type.
 * @member {object} [tags] Resource tags
 */
export interface ProxyResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  tags?: { [propertyName: string]: string };
}


/**
 * @class
 * Initializes a new instance of the LinkedServiceListResult class.
 * @constructor
 * The list linked service operation response.
 *
 */
export interface LinkedServiceListResult extends Array<LinkedService> {
}

/**
 * @class
 * Initializes a new instance of the DataSourceListResult class.
 * @constructor
 * The list data source by workspace operation response.
 *
 * @member {string} [nextLink] The link (url) to the next page of datasources.
 */
export interface DataSourceListResult extends Array<DataSource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the WorkspaceListUsagesResult class.
 * @constructor
 * The list workspace usages operation response.
 *
 */
export interface WorkspaceListUsagesResult extends Array<UsageMetric> {
}

/**
 * @class
 * Initializes a new instance of the WorkspaceListManagementGroupsResult class.
 * @constructor
 * The list workspace managmement groups operation response.
 *
 */
export interface WorkspaceListManagementGroupsResult extends Array<ManagementGroup> {
}

/**
 * @class
 * Initializes a new instance of the WorkspaceListResult class.
 * @constructor
 * The list workspaces operation response.
 *
 */
export interface WorkspaceListResult extends Array<Workspace> {
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Result of the request to list solution operations.
 *
 * @member {string} [nextLink] URL to get the next set of operation list
 * results if there are any.
 */
export interface OperationListResult extends Array<Operation> {
  readonly nextLink?: string;
}
