/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for a replica, containing information about the
 * data and the algorithm used by health store to evaluate health. The
 * evaluation is returned only when the aggregated health state is either Error
 * or Warning.
 *
 * @extends models['HealthEvaluation']
 */
class ReplicaHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a ReplicaHealthEvaluation.
   * @property {uuid} [partitionId] Id of the partition to which the replica
   * belongs.
   * @property {string} [replicaOrInstanceId] Id of a stateful service replica
   * or a stateless service instance. This ID is used in the queries that apply
   * to both stateful and stateless services. It is used by Service Fabric to
   * uniquely identify a replica of a partition of a stateful service or an
   * instance of a stateless service partition. It is unique within a partition
   * and does not change for the lifetime of the replica or the instance. If a
   * stateful replica gets dropped and another replica gets created on the same
   * node for the same partition, it will get a different value for the ID. If
   * a stateless instance is failed over on the same or different node it will
   * get a different value for the ID.
   * @property {array} [unhealthyEvaluations] List of unhealthy evaluations
   * that led to the current aggregated health state of the replica. The types
   * of the unhealthy evaluations can be EventHealthEvaluation.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ReplicaHealthEvaluation
   *
   * @returns {object} metadata of ReplicaHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Replica',
      type: {
        name: 'Composite',
        polymorphicDiscriminator: {
          serializedName: 'Kind',
          clientName: 'kind'
        },
        uberParent: 'HealthEvaluation',
        className: 'ReplicaHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            isPolymorphicDiscriminator: true,
            type: {
              name: 'String'
            }
          },
          partitionId: {
            required: false,
            serializedName: 'PartitionId',
            type: {
              name: 'String'
            }
          },
          replicaOrInstanceId: {
            required: false,
            serializedName: 'ReplicaOrInstanceId',
            type: {
              name: 'String'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ReplicaHealthEvaluation;
